/*
cmfengine.cpp - a point of sale client class
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>
#include <gdk/gdkkeysyms.h>

#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
//#include "ckeymap.h"
#include "cmfengine.h"

int key_snooper(GtkWidget *grab_widget, GdkEventKey *event, gpointer func_data)
{
   bhmfengine::CMfPosEngine *eng=(bhmfengine::CMfPosEngine *) func_data;
	/* the problem: key acceleration in gtk2 works a bit strange. I do not
	 * understand it completely. The following is in part the result of a
	 * long trial and error process. If you can explain why it works, please
	 * tell me.
	 * btw, do this only if main_window is the current window. otherwise, 
	 * keypad's 0,2,4,6,8 won't work in gtkentry etc. (e.g. found in prefs)
	 */
	
   if(event->type == GDK_KEY_RELEASE){
	   //g_print("key_snooper: %i %i %s\n", event->state, event->keyval, gdk_keyval_name(event->keyval)); 
      if(eng->KeyEvent(event)){
         //g_print(" - consumed - \n");
         return TRUE;
      }
   }
   /*
	if(((event->keyval != GDK_KP_2) && (event->keyval != GDK_KP_Down) &&
		(event->keyval != GDK_KP_4) && (event->keyval != GDK_KP_Left) &&
		(event->keyval != GDK_KP_6) && (event->keyval != GDK_KP_Right) &&
		(event->keyval != GDK_KP_8) && (event->keyval != GDK_KP_Up) &&
		(event->keyval != GDK_KP_0) && (event->keyval != GDK_KP_Insert)) ||
		(strcmp (gtk_widget_get_name (gtk_widget_get_toplevel(grab_widget)),
			"main_window") != 0))
			event->state &= ~GDK_MOD2_MASK;
	*/
	/* another problem: we have keyboard accelerators which are simple
	 * keypresses, e.g. "1", "2" but also "s" for the sin button. if the
	 * formula entry is active and user types a "s", user expects a s to
	 * be appended. moreover the same for special keys like backspace etc.
	 * unfortunately, accelerators have higher priority and can't be somehow
	 * blocked (they can? tell me!). therefore if formula_entry is active
	 * and it's a "simple" key press (is this the best solution?), then emit
	 * the signal directly and return TRUE to prevent any further 
	 * procession.
	 */
	/*
	if(formula_entry_is_active(grab_widget) && (event->type == GDK_KEY_PRESS)) {
		if((event->state == 0) || (event->state == GDK_SHIFT_MASK)) {
			gtk_widget_event (glade_xml_get_widget (main_window_xml, "formula_entry"), (GdkEvent *)event);
			return TRUE;
		}
	}
	*/
	return FALSE;
}

namespace bhmfengine {

CMfPosEngine::CMfPosEngine()
{
   status=MF_STATUS_UNCONNECTED;
   mode=MF_MODE_IDLE;
   bKeyRedirect=false;
   bKeyDisabled=false;
   bLookup=false;
   bPostCompletion=false;
   current_transaction=-1;
   current_index=-1;
   //tctype=BHTRANSACTION_TYPE_NONE;
   tcstatus=0;
   
   memset(&print_strings, 0, sizeof(print_strings));
   memset(&bc_prefix, 0, sizeof(GlobalPrefix));
   memset(&taxlist, 0, sizeof(TaxList));
   memset(&pricelist, 0, sizeof(PriceList));
   memset(&paylist, 0, sizeof(PayList));
   
   //bc_prefix.transaction_prefix=21;
   bc_prefix.weight_prefix=22;
   bc_prefix.quantity_prefix=23;
   bc_prefix.price_prefix=24;
   bc_prefix.voucher_prefix=25;
   bc_prefix.instore_prefix=26;
   
   ps2bcprefix=GDK_dollar;
   ps2bcpostfix=GDK_Linefeed; 
   
   bCustDispRevertTimer=false;
   cdisplaytype=0;
   bcdOpen=false;
   
   display=Gdk::Display::get_default();
   gtk_key_snooper_install(key_snooper, this);
}

CMfPosEngine::CMfPosEngine(Gtk::Main *m)
{
   status=MF_STATUS_UNCONNECTED;
   bKeyRedirect=false;
   bKeyDisabled=false;
   current_transaction=-1;
   current_index=-1;
   tcstatus=0;
   main=m;
   memset(&print_strings, 0, sizeof(print_strings));
   memset(&bc_prefix, 0, sizeof(GlobalPrefix));
   memset(&taxlist, 0, sizeof(TaxList));
   memset(&pricelist, 0, sizeof(PriceList));
   
    //bc_prefix.transaction_prefix=21;
   bc_prefix.weight_prefix=22;
   bc_prefix.quantity_prefix=23;
   bc_prefix.price_prefix=24;
   bc_prefix.voucher_prefix=25;
   bc_prefix.instore_prefix=26;
   
   ps2bcprefix=GDK_dollar;
   ps2bcpostfix=GDK_Linefeed; 
   
   bCustDispRevertTimer=false;
   bcdOpen=false;
   
   display=Gdk::Display::get_default();
   gtk_key_snooper_install(key_snooper, this);
   
}

CMfPosEngine::~CMfPosEngine()
{
}

bool CMfPosEngine::Close()
{
   //thread->join();
   return true;
}

int CMfPosEngine::LoadConfig(char *fn)
{
   int ret=config.Parse(fn);
   system_signal.emit(BHMF_EVENT_READCONFIG, NULL);
   return ret;
}

int CMfPosEngine::OpenNetwork()
{
   char              c[255];
   unsigned short    p=0;
   bool              b;
   int               target;
   bhpos::CBhDevice *dev;
   
   network=new CNetwork(main);
   strcpy(c, "");
   config.GetOption(MFENG_CONFIG_HOST, &c);
   config.GetOption(MFENG_CONFIG_PORT, &p);
   network->SetServer(AF_INET, c, p);
   strcpy(c, "");
   config.GetOption(MFENG_CONFIG_NETDEVICE, &c);
   network->SetDevice(c);
   
   if(network->Open() < 0){
      systemerror_signal.emit(BHMF_EVENT_OPENNET, NULL);
      return 1;
   }

   network->signal_msg().connect(sigc::mem_fun(*this, &CMfPosEngine::NetworkMessage));
   network->signal_ctrlmsg().connect(sigc::mem_fun(*this, &CMfPosEngine::NetworkCtrlMessage));
   network->signal_timeout().connect(sigc::mem_fun(*this, &CMfPosEngine::NetworkTimeout));
   
   system_signal.emit(BHMF_EVENT_OPENNET, NULL);
   return 0;
}

void CMfPosEngine::GetList(BhListType t, void *d)
{
   //MenuItemOptions
   
   switch(t){
   case BHLISTTYPE_TAX:
      memcpy(d, &taxlist, sizeof(int)+(sizeof(BhTaxListItem)*taxlist.numitems));
      break;
   case BHLISTTYPE_PRICE:
      memcpy(d, &pricelist, sizeof(int)+(sizeof(BhPriceListItem)*pricelist.num_items));
      break;
   case BHLISTTYPE_PAY:
      memcpy(d, &paylist, sizeof(PayList));
      break;
   case BHLISTTYPE_MENUHEADING:
      memcpy(d, &menuheadings, sizeof(MenuHeadings));
      break;
   case BHLISTTYPE_MENUITEM:
      memcpy(d, &menuitems, sizeof(MenuItems));
      break;
   }
}

void CMfPosEngine::SetStatus(BhMfStatus s) 
{
   status=s;
   //if(status==xxx);
}

int CMfPosEngine::Command(BhMfCommand c, void *d)
{
   switch(c){
   case BHMF_CMD_READCONFIG:
      return LoadConfig((char *) d);
   
   case BHMF_CMD_OPENNET:
      return OpenNetwork();
   
   case BHMF_CMD_CONNECT:
      return Connect();
   
   case BHMF_CMD_DISCONNECT:
      return Disconnect();
   
   case BHMF_CMD_PRINTERSETTINGS:
      status=MF_STATUS_PRNSETUP;
      return GetPrinterSettings();
   
   case BHMF_CMD_PREFIXLIST:
      return GetPrefixList();
   
   case BHMF_CMD_PRICELIST:
      return GetPriceList();
   
   case BHMF_CMD_TAXLIST:
      return GetTaxList();
   
   case BHMF_CMD_PAYLIST:
      return GetPayList();
   
   case BHMF_CMD_MENUHEADINGS:
      return GetMenuHeadings();
   
   case BHMF_CMD_MENUITEMS:
      return GetMenuItems(*((int *) d));
   
   case BHMF_CMD_HARDWARE:
      return Configure();
   
   case BHMF_CMD_LOGIN:
      return Login((BhLogonRequestEx *) d);
      break;

   case BHMF_CMD_LOGOUT:
      return Logout();
      break;
   
   case BHMF_CMD_SITEM_ADD:
      return SaleItemAdd(*((bhpos::CSaleItem *) d), true);
      break;
   
   case BHMF_CMD_SITEM_ADD_NLU:
      return SaleItemAdd(*((bhpos::CSaleItem *) d), false);
      break;
   
   case BHMF_CMD_SITEM_CASHADD:
      return SaleItemAdd(*((float *) d));
      break;
   
   case BHMF_CMD_SITEM_LOOKUP:
      return SaleItemLookup(*((bhpos::CSaleItem *) d));
      break;
   
   case BHMF_CMD_SITEM_EDIT:
      return SaleItemEdit(*((bhpos::CSaleItem *) d));
      break;
   
   case BHMF_CMD_SITEM_REMOVE:
      return SaleItemRemove(*((bhpos::CSaleItem *) d));
      break;
   
   case BHMF_CMD_SITEM_REMOVEBYSCAN:
      SaleItemRemoveByScan();
      break;
   
   case BHMF_CMD_PITEM_AUTH:
      return PayAuth(*((bhpos::CPayItem *) d));
      break;
   
   case BHMF_CMD_PITEM_ADD:
      return PayItemAdd(*((bhpos::CPayItem *) d));
      break;
   
   case BHMF_CMD_PITEM_EDIT:
      return PayItemEdit(*((bhpos::CPayItem *) d));
      break;
   
   case BHMF_CMD_PITEM_REMOVE:
      return PayItemRemove(*((bhpos::CPayItem *) d));
      break;
   
   case BHMF_CMD_PITEM_REMOVEALL:
      return PayItemRemoveAll();
      break;
   /*
   case BHMF_CMD_TRANS_SELECT:
      return TransSelect(*((int *) d));
      break;
   */
   case BHMF_CMD_TRANS_VOID:
      return TransVoid();
      break;
   
   case BHMF_CMD_TRANS_COMMIT:
      tcstatus=MF_STATUS_TRANSCOMMIT;
      return TransCommit();
      break;
   
   case BHMF_CMD_TRANS_NOSALE:
      return TransNoSale();
      break;
   
   case BHMF_CMD_TRANS_ID:
      return TransID();
      break;
   
   case BHMF_CMD_TRANS_TENDER:
      return TransTender();
      break;
   
   case BHMF_CMD_TRANS_CURRENT:
      transstack.SetHistory(false);
      break;
   
   case BHMF_CMD_TRANS_HISTORY:
      transstack.SetHistory(true);
      break;
   
   case BHMF_CMD_TRANS_HISTORY_CYCLE:
      break;
   
   case BHMF_CMD_TRANS_CUSTLOOKUP:
   case BHMF_CMD_TRANS_RENTALLOOKUP:
   case BHMF_CMD_TRANS_RENTALRETURN:
   case BHMF_CMD_TRANS_DELIVERYRETURNLOOKUP:
   case BHMF_CMD_TRANS_CUSTITEMPICKUP:
      break;
   
   case BHMF_CMD_SYSTEM_KEY:
      break;
   
   case BHMF_CMD_SYSTEM_POSTCOMP:
      bPostCompletion=true;
      break;
   
   case BHMF_CMD_SYSTEM_USERLOOKUP:
      break;
   
   case BHMF_CMD_SYSTEM_CUSTLOOKUP:
      break;
   
   case BHMF_CMD_SYSTEM_TRANSLIST:
      break;
   
   case BHMF_CMD_SYSTEM_BEEP:
   {
      int num=*((int *) d);
      if(display){
         while(num){
            display->beep();
            num--;
         }
      }
   }
      break;
   
   case BHMF_CMD_SYSTEM_PAYIN:
   {
      double a=*((double *) d);
      Adjustment(BHMSG_POST_ADJUST_PAYIN,a);
   }
      break;
   case BHMF_CMD_SYSTEM_PAYOUT:
   {
      double a=*((double *) d);
      Adjustment(BHMSG_POST_ADJUST_PAYOUT,a);
   }
      break;
   
   case BHMF_CMD_SYSTEM_PAYADJUSTMENT:
   {
      double a=*((double *) d);
      Adjustment(BHMSG_POST_ADJUST_CASHDRAWER,a);
   }
      break;
   
   case BHMF_CMD_HW_CUSTDISP_CLEAR:
      CustomerDisplayClear();
      break;
   
   case BHMF_CMD_HW_CUSTDISP_PRE:
      if(print_strings.custdisp_presale_size!=0){
         CustomerDisplay(print_strings.custdisp_presale);
      }
      break;
   
   case BHMF_CMD_HW_CUSTDISP_IDLE:
      if(print_strings.custdisp_idle_size!=0){
         CustomerDisplay(print_strings.custdisp_idle);
      }
      break;
      
   case BHMF_CMD_HW_CUSTDISP:
   {
      char *msg=(char *) d;
      CustomerDisplay(msg);
   }
      break;
   
   case BHMF_CMD_HW_CUSTDISP_LINE1:
   {
      char *msg=(char *) d;
      CustomerDisplay(1, false, msg);
   }
      break;
   
   case BHMF_CMD_HW_CUSTDISP_LINE2:
   {
      char *msg=(char *) d;
      CustomerDisplay(2, false, msg);
   }
      break;
   
   case BHMF_CMD_HW_CUSTDISP_TOTAL:
      transstack.SetStatus(current_transaction, bhpos::BHTRANS_STATUS_TOTAL);
      CustomerDisplayCmd(1);
      break;
   
   case BHMF_CMD_HW_CUSTDISP_TENDER:
      CustomerDisplayCmd(2);
      break;
   
   case BHMF_CMD_HW_CUSTDISP_CHANGE:
      CustomerDisplayCmd(3);
      break;
   
   case BHMF_CMD_HW_CUSTDISP_POST:
   {
      if(print_strings.custdisp_postsale_size!=0){
         CustomerDisplay(print_strings.custdisp_postsale);
      }
   }
      break;
   
   case BHMF_CMD_HW_PRINTER:
      break;
   
   case BHMF_CMD_HW_PRINTER_CHQLOAD:
      Print(bhpos::PRNCMD_CHQ_LOADTIMER, NULL, 0);
      break;
   
   case BHMF_CMD_HW_PRINTER_CHQPRINT:
      Print(bhpos::PRNCMD_CHQ_PRINT, NULL, 0);
      break;
   case BHMF_CMD_HW_PRINTER_CHQREMOVE:
      break;
   
   case BHMF_CMD_HW_CASHDRAWER:
   {
      int ft;
      if(d==NULL){
         ft=0;
      }
      else{
         ft=*((int *) d);
      }
      return CashDrawer(ft);
   }
      break;
   
   case BHMF_CMD_HW_CASHDRAWERTIMER:
      return CashDrawerTimer();
      
   default:
      // check any plug-ins
      return 1;
   }
   
   return 1;
}

int CMfPosEngine::Command(BhMfCommand cmd, void *d, void *n)
{
   switch(cmd){
   case BHMF_CMD_SITEM_EDIT:
      return SaleItemEdit(*((bhpos::CSaleItem *) d), *((int *) n));
      break;
   
   case BHMF_CMD_SITEM_REMOVE:
      return SaleItemRemove(*((bhpos::CSaleItem *) d), *((int *) n));
      break;
   
   case BHMF_CMD_PITEM_EDIT:
      return PayItemEdit(*((bhpos::CPayItem *) d), *((int *) n));
      break;
   
   case BHMF_CMD_PITEM_REMOVE:
      return PayItemRemove(*((bhpos::CPayItem *) d), *((int *) n));
      break;
   }
   return 0;
}

int CMfPosEngine::SetOption(BhMfConfig o, void *v)
{
   if(v==NULL) return -1;
   
   config.SetOption(o, v);
   
   return 0;
}

int CMfPosEngine::GetOption(BhMfConfig o, void *v)
{
   if(v==NULL) return -1;
   
   if(o==MFENG_CONFIG_KEYREDIRECTION){
      //v=((int (*)(GdkEventKey *)) &KeyEvent);
      return 0;
   }
   config.GetOption(o, v);
   
   return 0;
}

/*
int CMfPosEngine::InputStream(GdkEventAny *event)
{
   if(stream_manager){
      ret=stream_manager.Handler(event);
   }
   return ret;
}

int CMfPosEngine::DeviceStream(int source, GdkInputCondition condition)
{
   return 0;
}

int CMfPosEngine::EventHandler(BhEventAny *bhevent)
{
}
*/
int CMfPosEngine::ModuleHandler()
{
   return 0;
}

int CMfPosEngine::ScriptHandler()
{
   return 0;
}

int CMfPosEngine::Post(BhPosHdr *h, bool bResend)
{
   return network->Send(h, false, false, 0);
}

int CMfPosEngine::GetSystemInfo(BhMfSystemInfoType info, int b, void *a)
{
   switch(info){
   case BHSYSINFO_PREFIXLIST:
      *((GlobalPrefix *) a)=bc_prefix;
      break;
   case BHSYSINFO_PRICELIST:
      *((PriceList *) a)=pricelist;
      break;
   case BHSYSINFO_TAXLIST:
      *((TaxList *) a)=taxlist;
      break;
   case BHSYSINFO_PAYLIST:
      *((PayList *) a)=paylist;
      break;
   case BHSYSINFO_MENUHEADINGS:
      *((MenuHeadings *) a)=menuheadings;
      break;
   case BHSYSINFO_MENUITEMS:
      *((MenuItems *) a)=menuitems;
      break;
   case BHSYSINFO_MENUITEMOPTIONS:
      *((MenuItemOptions *) a)=menuitemoptions;
      break;
   case BHSYSINFO_PRINTERPARAM:
      break;
   }
}

int CMfPosEngine::GetTransInfo(BhMfTransInfoType info, int b, void *a)
{
   //mutex.lock();
   bool bServerSide;
   config.GetOption(MFENG_CONFIG_TRANSSERVERSIDE, &bServerSide);
   
   if(bServerSide){
      switch(info){
      case BHTRANSINFO_NUMTRANS:
         *((int *) a)=transstack.NumTrans();
         break;
   
      case BHTRANSINFO_TRANSID:
         *((int *) a)=transstack.GetTransID(b);
         break;
   
      case BHTRANSINFO_CURRENTIDX:
         *((int *) a)=current_index;
         break;
   
      case BHTRANSINFO_CURRENTNUMITEMS:
         if(current_transaction==-1) *((int *) a)=0;
         *((int *) a)=transstack.NumItems(current_transaction, bhpos::BHITEM_TYPE_QTY);
         break;
   
      case BHTRANSINFO_CURRENTSUBTOTAL:
         if(current_transaction==-1) *((double *) a)=0.0;
         *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_SUBTOTAL);
         break;
   
      case BHTRANSINFO_CURRENTTAX:
         if(current_transaction==-1) *((double *) a)=0.0;
         *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TAX);
         break;
   
      case BHTRANSINFO_CURRENTTOTAL:
         if(current_transaction==-1) *((double *) a)=0.0;
         *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TOTAL);
         break;
   
      case BHTRANSINFO_CURRENTSTATUS:
         *((bhpos::BhTransStatus *) a)=transstack.GetStatus(current_transaction);
         break;
   
      case BHTRANSINFO_STATUS:
         *((bhpos::BhTransStatus *) a)=transstack.GetStatus(b);
         break;
   
      case BHTRANSINFO_CURRENTTOTALTENDERED:
         if(current_transaction==-1) *((double *) a)=0.0;
         *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TOTALTENDERED);
         break;
   
      case BHTRANSINFO_CURRENTCHANGE:
         if(current_transaction==-1) *((double *) a)=0.0;
         *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_CHANGE);
         break;
      
    /*  
      case BHTRANSINFO_NUMITEMS:
         *((int *) a)=transstack.NumItems(b), *((bhpos::BhItemType *) b));
         break;
   
      case BHTRANSINFO_NUMBER:
         *((double *) a)=transstack.GetNumber(b), *((bhpos::BhTransNumber *) b));
         break;*/
      }
   
      //mutex.unlock();
      return 0;
   }
   
   
   switch(info){
   case BHTRANSINFO_NUMTRANS:
      *((int *) a)=transstack.NumTrans();
      break;
   
   case BHTRANSINFO_TRANSID:
      *((int *) a)=transstack.GetTransID(b);
      break;
   
   case BHTRANSINFO_CURRENTIDX:
      *((int *) a)=current_index;
      break;
   
   case BHTRANSINFO_CURRENTNUMITEMS:
      if(current_transaction==-1) *((int *) a)=0;
      *((int *) a)=transstack.NumItems(current_transaction, bhpos::BHITEM_TYPE_QTY);
      break;
   
   case BHTRANSINFO_CURRENTSUBTOTAL:
      if(current_transaction==-1) *((double *) a)=0.0;
      *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_SUBTOTAL);
      break;
   
   case BHTRANSINFO_CURRENTTAX:
      if(current_transaction==-1) *((double *) a)=0.0;
      *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TAX);
      break;
   
   case BHTRANSINFO_CURRENTTOTAL:
      if(current_transaction==-1) *((double *) a)=0.0;
      *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TOTAL);
      break;
   
   case BHTRANSINFO_CURRENTSTATUS:
      *((bhpos::BhTransStatus *) a)=transstack.GetStatus(current_transaction);
      break;
   
   case BHTRANSINFO_STATUS:
      *((bhpos::BhTransStatus *) a)=transstack.GetStatus(b);
      break;
   
   case BHTRANSINFO_CURRENTTOTALTENDERED:
      if(current_transaction==-1) *((double *) a)=0.0;
      *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_TOTALTENDERED);
      break;
   
   case BHTRANSINFO_CURRENTCHANGE:
      if(current_transaction==-1) *((double *) a)=0.0;
      *((double *) a)=transstack.GetNumber(current_transaction, bhpos::BHNUM_CHANGE);
      break;
   
   case BHTRANSINFO_NUMPAYITEMS:
      if(current_transaction==-1) *((int *) a)=0;
      *((int *) a)=transstack.NumPayItems(current_transaction);
      break;
 /*  
   case BHTRANSINFO_NUMITEMS:
      *((int *) a)=transstack.NumItems(b), *((bhpos::BhItemType *) b));
      break;
   
   case BHTRANSINFO_NUMBER:
      *((double *) a)=transstack.GetNumber(b), *((bhpos::BhTransNumber *) b));
      break;*/
   }
   
   //mutex.unlock();
   return 0;
}

}
