/*
cmfengine.h - a point of sale engine
Copyright (C) 2004  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
*/

#ifndef _MFPOSENGINE_H_
#define _MFPOSENGINE_H_

#include <queue>
#include <sigc++/sigc++.h>
#include <glibmm/random.h>
#include <glibmm/thread.h>
#include <glibmm/timer.h>
#include <glibmm/ustring.h>
#include <gdkmm/display.h>

#include <pos.h>
#include <ctransitems.h>
#include <cash.h>
#include <clocale.h>
#include <bhdev.h>
#include <bhnetwork.h>
#include <bhhardware.h>

#include "cmfplugin.h"
#include "cmfconfig.h"

/**
 * @defgroup mfengine MF Engine
 * @{
 */
 
 
namespace bhmfengine {
/**
 * @defgroup engine The Engine
 * @{
 *
 * @ingroup mfengine
 * @{
 */   
   
   
/*! 
 *  \brief The engine status
 */   
enum BhMfStatus{
   MF_STATUS_UNCONNECTED=0,
   MF_STATUS_CONNECTING,
   MF_STATUS_DISCONNECTING,
   MF_STATUS_PRNSETUP,
   MF_STATUS_PRNINIT,
   MF_STATUS_PRNPREPRINT,
   MF_STATUS_PRNHEADER,
   MF_STATUS_PRNITEM,
   MF_STATUS_PRNTRANSSUM,
   MF_STATUS_PRNPAYSUM, 
   MF_STATUS_PRNFOOTER, //10
   MF_STATUS_PRNPOSTPRINT,
   MF_STATUS_JRNPREPRINT,
   MF_STATUS_JRNITEM,
   MF_STATUS_JRNPOSTPRINT,
   MF_STATUS_LBLPREPRINT,
   MF_STATUS_LBLITEM,
   MF_STATUS_LBLPOSTPRINT,
   MF_STATUS_CHKPREPRINT,
   MF_STATUS_CHKITEM, 
   MF_STATUS_CHKPOSTPRINT,  //20
   MF_STATUS_SLIPPREPRINT,
   MF_STATUS_SLIPHEADER,
   MF_STATUS_SLIPITEM,
   MF_STATUS_SLIPFOOTER,
   MF_STATUS_SLIPPOSTPRINT,
   MF_STATUS_ENDPREPRINT,
   MF_STATUS_ENDITEM,
   MF_STATUS_ENDPOSTPRINT,
   MF_STATUS_MICRPREREAD, 
   MF_STATUS_MICRREAD, //30
   MF_STATUS_MICRSTATUS,
   MF_STATUS_MICRPOSTREAD,
   MF_STATUS_CUSTDPRESALE,
   MF_STATUS_CUSTDPOSTSALE,
   MF_STATUS_CUSTDIDLE,
   MF_STATUS_TAXLIST,
   MF_STATUS_PRICELIST,
   MF_STATUS_PREFIXLIST,
   MF_STATUS_PAYLIST,   
   MF_STATUS_MENUHEADINGS, //40
   MF_STATUS_MENUITEMS,
   MF_STATUS_LOGGEDOUT,
   MF_STATUS_PASSIVELOGOUT,
   MF_STATUS_USERLOOKUP,
   MF_STATUS_LOGGEDIN,
   MF_STATUS_IDLE,
   MF_STATUS_PRICESELECT,
   MF_STATUS_TRANSID,
   MF_STATUS_TRANSIDONLY, 
   MF_STATUS_TRANSIDADD, //50
   MF_STATUS_TRANSAUTH,
   MF_STATUS_SCANNED, 
   //MF_STATUS_KEYPRESS,
   MF_STATUS_QUANTITY,
   MF_STATUS_PLUWAIT,
   MF_STATUS_PLU,
   MF_STATUS_PLUSCAN,
   MF_STATUS_PLULOOKUP,
   MF_STATUS_DESCRIPTIONENTRY, 
   MF_STATUS_PRICEENTRY, 
   MF_STATUS_CASHITEM, // 60
   MF_STATUS_LISTEDIT, 
   MF_STATUS_PLULOOKUPNA,
   MF_STATUS_SCANREMOVE,  
   MF_STATUS_LOGGOUTACK, 
   MF_STATUS_VOIDACK,
   MF_STATUS_PAYEDIT,
   MF_STATUS_CONFIRMCOMMIT,
   
   MF_STATUS_PAYAUTH, 
   MF_STATUS_TENDER, 
   MF_STATUS_PAYTYPE, //70
   MF_STATUS_PAYAMOUNT,
   
   MF_STATUS_CUSTLOOKUP,
   MF_STATUS_CUSTNEW,
   MF_STATUS_CUSTADD,
   MF_STATUS_CUSTEDIT,
   MF_STATUS_CUSTCHARGEBAL,
   MF_STATUS_CUSTDROPIN,
   MF_STATUS_CUSTPICKUP, 
   MF_STATUS_CUSTDELIVERY,
   MF_STATUS_CUSTRENTAL, // 80
   MF_STATUS_CUSTSUMMARY,
   MF_STATUS_CUSTLAYBY,
   
   MF_STATUS_RENTALLIST,
   MF_STATUS_RENTALLOOKUP,
   MF_STATUS_RENTALRETURN,
   MF_STATUS_DELIVERYRETLIST,
   MF_STATUS_DELIVERYRETURN,
   MF_STATUS_CUSTITEMPICKUP,
   
   MF_STATUS_PAYIN,
   MF_STATUS_PAYOUT, // 90
   MF_STATUS_PAYADJUSTMENT,
   MF_STATUS_ADJUSTMENT,
   MF_STATUS_DISCREPANCY,
   MF_STATUS_NOSALE,
   
   MF_STATUS_RETURNREASON,
   MF_STATUS_CASHBACKCHECK,
   
   MF_STATUS_PRINTING,
   MF_STATUS_CDOPEN,
   MF_STATUS_TRANS,
   MF_STATUS_UNKNOWNPLU, //100
   MF_STATUS_ACK, 
   MF_STATUS_MAGWAIT,
   MF_STATUS_SMARTWAIT,
   MF_STATUS_MICRWAIT,
   MF_STATUS_CHEQUELOADWAIT,
   MF_STATUS_CHEQUEPRNWAIT,
   MF_STATUS_CHEQUEREMOVEWAIT,
   
   MF_STATUS_TRANSCOMMIT,
   MF_STATUS_PLUCOMMIT,
   MF_STATUS_RETURNPLUCOMMIT,
   MF_STATUS_RENTALPLUCOMMIT, 
   MF_STATUS_LAYBYPLUCOMMIT, 
   MF_STATUS_PRICECOMMIT, //110
   MF_STATUS_UPLUCOMMIT, 
   MF_STATUS_PAYCOMMIT,
   MF_STATUS_TRANSEND,
   MF_STATUS_TRANSLIST,
   
   MF_STATUS_CASHADJUST,
   MF_STATUS_TRANSRETRIEVE,
   MF_STATUS_PLURETRIEVE,
   MF_STATUS_UPLURETRIEVE,
   MF_STATUS_PAYRETRIEVE, 
   MF_STATUS_TRANSRETEND, 
   MF_STATUS_TRANSRETEDIT, //120
   MF_STATUS_CUSTENTRYOPEN, 
   MF_STATUS_TRANSHISTORY,
   MF_STATUS_BLINDBALANCEFETCH,
   
   MF_STATUS_TRANSSUM,
   MF_STATUS_USERSUM,
   MF_STATUS_CASHDACK,
   MF_STATUS_DELIVERYLIST,
   MF_STATUS_BLINDBALANCE,
   MF_STATUS_ZTAPE,
   MF_STATUS_IFACK,
   
   MF_STATUS_SLIPWAIT,
   MF_STATUS_CHEQUEWAIT,
   MF_STATUS_PRINTWAIT
};

/*
 *! Mode
 *  \brief The engine mode.
 */  
enum BhMfMode{
   MF_MODE_IDLE, /*!< Idle */
   MF_MODE_SALE  /*!< Sale */
};

/*
#define POST_PTFLAGS_BASE           1
#define POST_PTFLAGS_SERVSHUTDOWN   (POST_PTFLAGS_BASE << 0)
#define POST_PTFLAGS_KEYMAPDROP     (POST_PTFLAGS_BASE << 1)
#define POST_PTFLAGS_KEYMAP         (POST_PTFLAGS_BASE << 2)
#define POST_PTFLAGS_KEYMAPADD      (POST_PTFLAGS_BASE << 3)
#define POST_PTFLAGS_KEYMAPEDIT     (POST_PTFLAGS_BASE << 4)
#define POST_PTFLAGS_KEYMAPDELETE   (POST_PTFLAGS_BASE << 5)

#define POST_MODE_UNKNOWN           0
#define POST_MODE_BOOTING           1
#define POST_MODE_NOUSER            2
#define POST_MODE_IDLE              3
#define POST_MODE_SALE              4
#define POST_MODE_SUPERVISOR        5
#define POST_MODE_RETURNS           6
#define POST_MODE_GOODSIN           7
#define POST_MODE_TRANSEDIT	      8

#define POWERFAIL_POLICY1     1
#define POWERFAIL_POLICY2     2
#define POWERFAIL_POLICY3     3
#define POWERFAIL_POLICY4     4
#define POWERFAIL_POLICY5     5

#define PRINT_CHEQUE_VALIDATE 1
#define PRINT_CHEQUE_PRINT    2
#define PRINT_SLIP            3

#define SIGPAD_SIGNTYPE_PRIVACY    1
#define SIGPAD_SIGNTYPE_RX         2
#define SIGPAD_SIGNTYPE_SAFETYCAPS 3
#define SIGPAD_SIGNTYPE_COUNSELING 4
#define SIGPAD_SIGNTYPE_CREDIT     5
#define SIGPAD_SIGNTYPE_DEBIT      6
*/

/*! BhMfCommand.
 * \brief The engine comands.
 * Commands that the mfengine accepts.
 */  
enum BhMfCommand{
   BHMF_CMD_READCONFIG,       //! Read configuration file.
   BHMF_CMD_OPENNET,          //! Open network.
   BHMF_CMD_CONNECT,          //! Connect to server.
   BHMF_CMD_DISCONNECT,       //! Disconnect from server.
   BHMF_CMD_SHUTDOWN,         //! Shutdown.
   BHMF_CMD_REBOOT,
   BHMF_CMD_RESTART,
   BHMF_CMD_SLEEP,
   BHMF_CMD_PRINTERSETTINGS,  //! Fetch printer settings.
   BHMF_CMD_PREFIXLIST,       //! Fetch barcode prefixes.
   BHMF_CMD_PRICELIST,        //! Fetch price list.
   BHMF_CMD_TAXLIST,          //! Fetch tax list.
   BHMF_CMD_PAYLIST,          //! Fetch pay types list.
   BHMF_CMD_MENUHEADINGS,     //! Fetch menu headings.
   BHMF_CMD_MENUITEMS,        //! Fetch menu items.
   BHMF_CMD_HARDWARE,         //! Setup hardware.
   BHMF_CMD_USERLOOKUP,       //! User lookup.
   BHMF_CMD_LOGIN,            //! User login.
   BHMF_CMD_LOGOUT,           //! User logout.
   BHMF_CMD_PLUSCAN,          //! PLU scan.
   BHMF_CMD_SITEM_SCAN,       //! PLU lookup.
   BHMF_CMD_SITEM_ADD,
   BHMF_CMD_SITEM_ADD_NLU,
   BHMF_CMD_SITEM_CASHADD,    //! Add a cash item.
   BHMF_CMD_SITEM_LOOKUP,     //! PLU lookup.
   BHMF_CMD_SITEM_RETURN,
   BHMF_CMD_SITEM_EDIT,
   BHMF_CMD_SITEM_REMOVE,
   BHMF_CMD_SITEM_REMOVEBYSCAN,
   BHMF_CMD_PITEM_AUTH,
   BHMF_CMD_PITEM_ADD,
   BHMF_CMD_PITEM_EDIT,
   BHMF_CMD_PITEM_REMOVE,
   BHMF_CMD_PITEM_REMOVEALL,
   //BHMF_CMD_TRANS_SELECT,        //! Select transaction from list.
   BHMF_CMD_TRANS_CYCLE,         //! Cycle through transaction list.
   BHMF_CMD_TRANS_VOID,          //! Void current transaction.
   BHMF_CMD_TRANS_COMMIT,        //! Commit current transaction.
   BHMF_CMD_TRANS_NOSALE,        //! Fire cash drawer.
   BHMF_CMD_TRANS_ID,            //! Get a transaction id.
   BHMF_CMD_TRANS_IDONLY,
   BHMF_CMD_TRANS_TENDER,
   BHMF_CMD_TRANS_END,
   BHMF_CMD_TRANS_CURRENT,       //! Set transaction to current list.
   BHMF_CMD_TRANS_HISTORY,       //! Set transaction to history list.
   BHMF_CMD_TRANS_HISTORY_CYCLE, //! Cycle through transaction history list.
   
   BHMF_CMD_TRANS_CUSTLOOKUP,
   BHMF_CMD_TRANS_RENTALLOOKUP,
   BHMF_CMD_TRANS_RENTALRETURN,
   BHMF_CMD_TRANS_DELIVERYRETURNLOOKUP,
   BHMF_CMD_TRANS_CUSTITEMPICKUP,
   
   BHMF_CMD_SYSTEM_NUMTRANS,
   BHMF_CMD_SYSTEM_TRANSLIST,
   BHMF_CMD_SYSTEM_KEY,
   BHMF_CMD_SYSTEM_POSTCOMP,
   BHMF_CMD_SYSTEM_USERLOOKUP,
   BHMF_CMD_SYSTEM_CUSTLOOKUP,
   BHMF_CMD_SYSTEM_BEEP,
   BHMF_CMD_SYSTEM_BLINDBALANCE,
   BHMF_CMD_SYSTEM_ZTAPE,
   BHMF_CMD_SYSTEM_PAYIN,
   BHMF_CMD_SYSTEM_PAYOUT,
   BHMF_CMD_SYSTEM_PAYADJUSTMENT,
   BHMF_CMD_HW_CUSTDISP_CLEAR,   //! customer display clear.
   BHMF_CMD_HW_CUSTDISP_PRE,     //! customer display pre.
   BHMF_CMD_HW_CUSTDISP,         //! customer display raw.
   BHMF_CMD_HW_CUSTDISP_IDLE,    //! customer display idle message.
   BHMF_CMD_HW_CUSTDISP_POST,    //! customer display post.
   BHMF_CMD_HW_CUSTDISP_LINE1,   //! customer display line 1.
   BHMF_CMD_HW_CUSTDISP_LINE2,   //! customer display line 2.
   BHMF_CMD_HW_CUSTDISP_TOTAL,   //! customer display total.
   BHMF_CMD_HW_CUSTDISP_TENDER,  //! customer display tender.
   BHMF_CMD_HW_CUSTDISP_CHANGE,  //! customer display change.
   BHMF_CMD_HW_PRINTER,          //! printer command.
   BHMF_CMD_HW_PRINTERCANCEL,    //! printer cancel command.
   BHMF_CMD_HW_PRINTER_CHQLOAD,  //! load cheque.
   BHMF_CMD_HW_PRINTER_CHQPRINT, //! print cheque.
   BHMF_CMD_HW_PRINTER_CHQREMOVE,//! eject cheque.
   BHMF_CMD_HW_CASHDRAWER,       //! Fire cash drawer.
   BHMF_CMD_HW_CASHDRAWERTIMER   //! set cash drawer timer.
};
/*! \enum BhMfEvent
    \brief The engine events.
    Events emitted by the mfengine.
*/  
enum BhMfEvent{
   BHMF_EVENT_READCONFIG,        //! configuration file read.
   BHMF_EVENT_OPENNET,           //! network opened.
   BHMF_EVENT_CONNECT,           //! connected to server.
   BHMF_EVENT_DISCONNECT,        //! Disconnect event.
   BHMF_EVENT_SHUTDOWN,          //! Shutdown event.
   BHMF_EVENT_REBOOT,
   BHMF_EVENT_RESTART,
   BHMF_EVENT_SLEEP,
   BHMF_EVENT_PRINTERSETTINGS,
   BHMF_EVENT_PREFIXLIST,
   BHMF_EVENT_PRICELIST,
   BHMF_EVENT_TAXLIST,
   BHMF_EVENT_PAYLIST,
   BHMF_EVENT_MENUHEADINGS,
   BHMF_EVENT_MENUITEMS,
   BHMF_EVENT_HARDWARE,
   BHMF_EVENT_USERLOOKUP,
   BHMF_EVENT_LOGIN,
   BHMF_EVENT_LOGOUT,
   BHMF_EVENT_IDLE,
   BHMF_EVENT_PLUSCAN,
   BHMF_EVENT_SITEM_SCAN,
   BHMF_EVENT_SITEM_LOOKUP,
   BHMF_EVENT_SITEM_PLUSCAN,
   BHMF_EVENT_SITEM_REMOVE,
   BHMF_EVENT_SITEM_EDIT,
   BHMF_EVENT_SITEM_RETURN,
   BHMF_EVENT_SITEM_UPDATE,
   BHMF_EVENT_PITEM_AUTH,
   BHMF_EVENT_PITEM_AUTHRETURN,
   BHMF_EVENT_SYSTEM_NUMTRANS,
   BHMF_EVENT_SYSTEM_TRANSLIST,
   BHMF_EVENT_TRANS_CYCLE,
   BHMF_EVENT_TRANS_SELECT,
   BHMF_EVENT_TRANS_VOID,
   BHMF_EVENT_TRANS_COMMIT,
   BHMF_EVENT_TRANS_ID,
   BHMF_EVENT_TRANS_IDONLY,
   BHMF_EVENT_TRANS_IDADD,
   BHMF_EVENT_TRANS_END,
   BHMF_EVENT_TRANS_LIST,
   BHMF_EVENT_SYSTEM_KEY,
   BHMF_EVENT_SYSTEM_USERLOOKUP,
   BHMF_EVENT_SYSTEM_CUSTLOOKUP,
   BHMF_EVENT_SYSTEM_PAYIN,
   BHMF_EVENT_SYSTEM_PAYOUT,
   BHMF_EVENT_SYSTEM_PAYADJUSTMENT,
   
   BHMF_EVENT_CUSTLOOKUP_SCAN,
   BHMF_EVENT_RENTALLOOKUP_SCAN,
   BHMF_EVENT_RENTALRETURN_SCAN,
   BHMF_EVENT_DELIVERYRETURNLOOKUP_SCAN,
   BHMF_EVENT_CUSTITEMPICKUP_SCAN,
   BHMF_EVENT_CUSTLOOKUP,
   BHMF_EVENT_RENTALLOOKUP,
   BHMF_EVENT_RENTALRETURN,
   BHMF_EVENT_DELIVERYRETURNLOOKUP,
   BHMF_EVENT_CUSTITEMPICKUP,
   
   BHMF_EVENT_HW_WAITINGONSLIP,
   BHMF_EVENT_HW_SLIPDONE,
   BHMF_EVENT_HW_CHEQUELOAD,
   BHMF_EVENT_HW_CHEQUEREMOVED,
   BHMF_EVENT_HW_CHEQUECANCELED,
   BHMF_EVENT_HW_WAITINGONMICR,
   BHMF_EVENT_HW_MICRDONE,
   BHMF_EVENT_HW_CASHDRAWEROPEN,
   BHMF_EVENT_HW_CASHDRAWERCLOSED,
   BHMF_EVENT_HW_WAITINGONSWIPE,
   BHMF_EVENT_HW_WAITINGONSMART,
   
   BHMF_EVENT_HWERROR_BCADD,
   BHMF_EVENT_HWERROR_BCOPEN,
   BHMF_EVENT_HWERROR_BCDEVICE,
   BHMF_EVENT_HWERROR_PRINTERADD,
   BHMF_EVENT_HWERROR_PRINTEROPEN,
   BHMF_EVENT_HWERROR_CASHDADD,
   BHMF_EVENT_HWERROR_CASHDOPEN
};

/*!  BhLoginType.
     \brief The login types.
*/  
enum BhLoginType{
   BHLOGIN_TYPE_USERPASSWORD, //!< username/password login.
   BHLOGIN_TYPE_PASSIVECODE,  //!< passive code login.
   BHLOGIN_TYPE_BARCODE,      //!< barcode login.
   BHLOGIN_TYPE_BARCODEIF,    //!< barcode interface login.
   BHLOGIN_TYPE_MAGSWIPE      //!< magswipe login.
};

enum BhMfPluType{
   BHPLU_TYPE_ITEM,
   BHPLU_TYPE_WEIGHT,
   BHPLU_TYPE_QUANTITY,
   BHPLU_TYPE_PRICE,
   BHPLU_TYPE_INTERFACE,
   BHPLU_TYPE_TOKEN,
   BHPLU_TYPE_VOUCHER,
   BHPLU_TYPE_CUSTOMER,
   BHPLU_TYPE_INSTORE
};

/*! Lookup Type
    \brief The lookup type.
*/  
enum BhLookupType{
   BHLOOKUP_TYPE_CURRITEM
};

/*! System Information Type
    \brief The system information types.
*/  
enum BhMfSystemInfoType{
   BHSYSINFO_PREFIXLIST,         //!< prefix list.
   BHSYSINFO_PRICELIST,
   BHSYSINFO_TAXLIST,
   BHSYSINFO_PAYLIST,
   BHSYSINFO_MENUHEADINGS,
   BHSYSINFO_MENUITEMS,
   BHSYSINFO_MENUITEMOPTIONS,
   BHSYSINFO_PRINTERPARAM
};

/*!\brief BhMfTransInfoType.
 *  The transaction information types.
 */  
enum BhMfTransInfoType{
   BHTRANSINFO_NUMTRANS,         //!< number of transactions.
   BHTRANSINFO_TRANSID,
   BHTRANSINFO_CURRENTIDX,
   BHTRANSINFO_CURRENTNUMITEMS,
   BHTRANSINFO_CURRENTSUBTOTAL,
   BHTRANSINFO_CURRENTTAX,
   BHTRANSINFO_CURRENTTOTAL,
   BHTRANSINFO_CURRENTSTATUS,
   BHTRANSINFO_CURRENTTOTALTENDERED,
   BHTRANSINFO_CURRENTCHANGE,
   BHTRANSINFO_STATUS,
   BHTRANSINFO_NUMITEMS,
   BHTRANSINFO_NUMBER,
   BHTRANSINFO_SUBTOTAL,
   BHTRANSINFO_TAX,
   BHTRANSINFO_TOTAL,
   BHTRANSINFO_NUMPAYITEMS
};

/*! Hardware events
    \brief etc.
*/
enum BhHardwareEvent{
   BHMF_HW_KB_KEYSTROKE,
   BHMF_HW_KB_MAGSWIPE
};

/*! Hardware errors
    \brief Hardware errors.
*/
enum HardwareError{
   ERR_HW_DEV_S0_OPEN,
   ERR_HW_DEV_S1_OPEN,
   ERR_HW_DEV_S2_OPEN,
   ERR_HW_DEV_S3_OPEN,
   ERR_HW_DEV_lP0_OPEN,
   ERR_HW_DEV_lP1_OPEN,
   ERR_HW_DEV_USB_OPEN,
   ERR_HW_PRINTER_NOTFOUND,
   ERR_HW_MAGSWIPE_TRACK1_READ,
   ERR_HW_MAGSWIPE_TRACK2_READ,
   ERR_HW_MAGSWIPE_TRACK3_READ
};

/*! Customer events
    \brief Customer events.
*/
enum BhCustomerEvent{
   BHMF_CUSTOMER_BARCODE,
   BHMF_CUSTOMER_MAGSWIPE,
   BHMF_CUSTOMER_DETAILS,
   BHMF_CUSTOMER_ADD,
   BHMF_CUSTOMER_EDIT,
   BHMF_CUSTOMER_CHARGEBAL,
   BHMF_CUSTOMER_DROPIN,
   BHMF_CUSTOMER_PICKUP,
};

/*! \enum BhListType
    \brief list types.
*/
enum BhListType{
   BHLISTTYPE_TAX,
   BHLISTTYPE_PRICE,
   BHLISTTYPE_PAY,
   BHLISTTYPE_MENUHEADING,
   BHLISTTYPE_MENUITEM
};

enum BhMfEmitType{
   BHMFEMITTYPE_NETWORK,
   BHMFEMITTYPE_NETWORKCTRL,
   BHMFEMITTYPE_NETWORKERROR,
   BHMFEMITTYPE_SYSTEM,
   BHMFEMITTYPE_TRANSACTION,
   BHMFEMITTYPE_POS,
   BHMFEMITTYPE_HARDWARE,
   BHMFEMITTYPE_SYSTEMERROR,
   BHMFEMITTYPE_TRANSACTIONERROR,
   BHMFEMITTYPE_POSERROR,
   BHMFEMITTYPE_HARDWAREERROR,
};

/*! 
\brief A struct for a tax rate list.*/
typedef struct tagTaxList{
   int            numitems;
   BhTaxListItem  *item;
}TaxList;

/*! 
\brief A struct for a price list.*/
typedef struct tagPriceList{
   int             num_items;
   BhPriceListItem *item;
}PriceList;

/*! 
\brief A struct for a pay type list.*/
typedef struct tagPayList{
   int           numitems;
   BhExPaytype  *item;
}PayList;

/*! 
\brief A struct for menu headings.*/
typedef struct tagMenuHeadings{
   int            numitems;
   BhMenuHeading *item;
}MenuHeadings;

/*! 
\brief A struct for menu items.*/
typedef struct tagMenuItems{
   int         numitems;
   BhMenuItem *item;
}MenuItems;

/*! 
\brief A struct for menu item options.*/
typedef struct tagMenuItemOption{
   int               numitems;
   BhMenuItemOption *item;
}MenuItemOptions;

/*! \brief The engine. */
class CMfPosEngine : public sigc::trackable
{
private:
   //! \enum status;
   BhMfStatus status;
   BhMfMode   mode;

   // command line params
   bool  bNoLogin;
   char *userlogin;
   char *password;

   char  userName[32];
   int   userID;

   // input stream stuff - to be moved to input manager
   bool bKeyDisabled;
   bool bKeyRedirect;
   bool bPostCompletion;
   int  ps2bcprefix;
   int  ps2bcpostfix;

   int                current_transaction;
   int                current_index;
   //BhTransactionType  tctype;
   int                tcstatus;
   bhpos::CTransStack transstack;
   
   char *net_config;
   char *input_config;
   char *db_config;

   plugin::CMfPlugInManager  plugin;
   //script::CScriptManager  scripts;
   CMfConfig                 config;
   CNetwork                 *network;
   bhpos::CHardwareManager   hardware;
   bhpos::CCash              cashdrawer;
   BhPosConnectReply         connection_info;
   bool                      bcdOpen;
   bool                      bLookup;
   //BhMenuHeadingList      *menuheadinglist;
   //BhMenuItemList         *menuitemlist;
   
   bool            bUseGlobalPrintSettings;
   GlobalPrint     print_strings;
   GlobalPrefix    bc_prefix;
   TaxList         taxlist;
   PriceList       pricelist;
   PayList         paylist;
   MenuHeadings    menuheadings;
   MenuItems       menuitems;
   MenuItemOptions menuitemoptions;
   //PostGUI     *pgui;
   //BhCustomerCharBalRet chargebal;

protected:
   Glib::RefPtr<Gdk::Display> display;
   //void threadfunc();
   void NetworkMessage(int, void *);
   void NetworkCtrlMessage(int, void *);
   void NetworkTimeout();   

   int ProcessCtrl(BhMcPosMsg *);
   int CtrlSystem(BhMcPosMsg *);
   int CtrlMessage(BhMcPosMsg *);
   int CtrlClient(BhMcPosMsg *);
   int CtrlMenu(BhMcPosMsg *);
   
   int ServerMessage(BhPosHdr *);

   int PostMessage(BhPosHdr *hdr);

   int SystemMessage(BhPosHdr *);
   int SystemConnect(BhPosHdr *);
   int SystemVersion(BhPosHdr *);
   int SystemAuthenticate(BhPosHdr *);
   int SystemShutdown(BhPosHdr *);
   int SystemService(BhPosHdr *);
   int SystemLogin(BhPosHdr *);
   int SystemInterface(BhPosHdr *);
   int SystemMsg(BhPosHdr *);
   int SystemQuery(BhPosHdr *);
   int SystemTime(BhPosHdr *);
   int SystemStatistics(BhPosHdr *);
   
   bool KeyInState(GdkEventKey *e);
   bool IsNumber(GdkEventKey *e);
   bool IsAlpha(GdkEventKey *e);
   bool BarcodeType(Glib::ustring barcode);
   bool IsMonetary(GdkEventKey *e);
   int  SwipeParse(bhpos::MagSwipeTrack t, const char *data, BhMagswipe *ms);
   //int Hardware(GdkEventKey *ev);
   //int   userID;
   //int   interfaceID;
   
   Glib::Mutex         mutex;
   Glib::Cond          condition;
   //CMessageQueue       queue;
   //Glib::Thread *const thread;
   Gtk::Main          *main;
   
   sigc::signal<void, int, void *> net_signal;
   sigc::signal<void, int, void *> netctrl_signal;
   sigc::signal<void, int>         nettimeout_signal;
   sigc::signal<void, int>         neterror_signal;
   sigc::signal<void, int, void *> system_signal;
   sigc::signal<void, int, void *> transaction_signal;
   sigc::signal<void, int, void *> pos_signal;
   sigc::signal<void, int, void *> hw_signal;
   sigc::signal<void, int, void *> systemerror_signal;
   sigc::signal<void, int, void *> transactionerror_signal;
   sigc::signal<void, int, void *> poserror_signal;
   sigc::signal<void, int, void *> hwerror_signal;
   
   int  LoadConfig(char *);
   int  OpenNetwork();
   int  Configure();
   int  Connect();
   int  Disconnect();
   int  GetPrinterSettings();
   int  GetPrefixList();
   int  GetPriceList();
   int  GetTaxList();
   int  GetMenuHeadings();
   int  GetMenuItems(int);
   int  GetPayList();
   int  SetupHardware();
   
   // login, etc
   int  Login(BhLogonRequestEx *);
   int  Logout();
   void Clockin(BhClockIn *);
   
   BhMfPluType BarcodeType(bhpos::CSaleItem& si);
   int  SaleItemAdd(bhpos::CSaleItem& si, bool);
   int  SaleItemAdd(float& p);
   int  SaleItemLookup();
   int  SaleItemLookup(bhpos::CSaleItem& si);
   int  SaleItemEdit(bhpos::CSaleItem& si);
   int  SaleItemEdit(bhpos::CSaleItem& si, int);
   int  SaleItemRemove(bhpos::CSaleItem& si);
   int  SaleItemRemove(bhpos::CSaleItem& si, int);
   void SaleItemRemoveByScan();
   int  PayAuth(bhpos::CPayItem& si);
   int  PayItemAdd(bhpos::CPayItem& si);
   int  PayItemEdit(bhpos::CPayItem& pi);
   int  PayItemEdit(bhpos::CPayItem& pi, int);
   int  PayItemRemove(bhpos::CPayItem& pi);
   int  PayItemRemove(bhpos::CPayItem& pi, int);
   int  PayItemRemoveAll();
   int  NumTrans();
   int  TransList();
   //int  TransSelect(int);
   int  TransVoid();
   int  TransCommit();
   int  TransNoSale();
   int  TransID();
   int  TransTender();
   int  EphemeralCommit();
   int  CommitPlu();
   int  CommitReturnPlu();
   int  CommitRentalPlu();
   int  CommitLaybyPlu();
   int  CommitUPlu();
   int  CommitPricePlu();
   
   void ReturnReason();
   void CashbackCheck(int type, double amount);
   
   void Adjustment(int type, double amount);
   void Discrepency(int type, double amount);
   void ztape();
   void GetTransSummary();
   void GetUserSummary();
   void BlindBalance(int flags);
   void Interface(int type, const char *data);
   
   // customer stuff
   void CustomerLookup(BhCustomerLookup *cs);
   void DeliveryList(int delID);
   void DeliveryReturn(int delID);
   void SigpadSign(int type, bool bSign);
   void CustomerAdd(BhCustomerItem *ci);
   void CustomerChargeBalance(BhCustomerCharBal *cb);
   void CustomerDropIn(BhCustomerItem *ci);
   void CustomerPickUp(BhCustomerItem *ci);
   void RentalList();
   void RentalLookup();
   void RentalReturnScan();
   
   void Print(bhpos::BhPrintType type, void *data, int len);
   
   sigc::connection cdisptimer_slot;
   bool bCustDispRevertTimer;
   int  cdisplaytype;
   bhpos::CSaleItem custdispSItem;
   bhpos::CPayItem  custdispPItem;
   bhpos::TransSummary transsum;
   
   void CustomerDisplaySetTimer(bool);
   int  CustomerDisplayTimer();
   void CustomerDisplayClear();
   void CustomerDisplayCmd(int);
   int  CustomerDisplay(char *msg);
   int  CustomerDisplay(int line, bool clear, char *msg);
   int  CustomerDisplay(bhpos::CSaleItem&);
   int  CustomerDisplay(bhpos::CPayItem&);
   
   int CashDrawerTimer();
   
   /*! \brief  Fire Cash Drawer 
       \param type: 0 - simple fire, 1 fire with close signal, 2 fire with open and close signal */
   int CashDrawer(int type);
   
   int  CustomerEvent(BhCustomerEvent type, void *ev);
   
   bhpos::CSaleItem   CurrPluLookup;
   bhpos::CPayItem    CurrPayAuth;

   sigc::connection keysnoop_conn;
   int  KeySnooperEvent(Gtk::Widget *widget, GdkEventAny *event);
   
public:
   CMfPosEngine();
   CMfPosEngine(Gtk::Main *);
   ~CMfPosEngine();
   bool Close();
   int  ModuleHandler();
   int  ScriptHandler();
   BhMfStatus GetStatus() {return status; }
   void SetStatus(BhMfStatus s);
   void SetDiplay(Glib::RefPtr<Gdk::Display> d) {display=d; }
   void GetList(BhListType, void *);
   
   /*! \brief  network signal */
   sigc::signal<void, int, void *> signal_net(){ return net_signal; }
   /*! \brief  network control signal (multicast) */
   sigc::signal<void, int, void *> signal_netctrl(){ return netctrl_signal; }
   /*! \brief  network timeout signal */
   sigc::signal<void, int> signal_nettimeout(){ return  nettimeout_signal; }
   /*! \brief  network error signal */
   sigc::signal<void, int> signal_neterror(){ return neterror_signal; }
   
   /*! \brief  system event signal */
   sigc::signal<void, int, void *> signal_system(){ return system_signal; }
   /*! \brief  transaction event signal */
   sigc::signal<void, int, void *> signal_transaction(){ return transaction_signal; }
   /*! \brief  pos event signal */
   sigc::signal<void, int, void *> signal_pos(){ return pos_signal; }
   
   /*! \brief  system error signal */
   sigc::signal<void, int, void *> signal_systemerror(){ return systemerror_signal; }
   /*! \brief  transaction error signal */
   sigc::signal<void, int, void *> signal_transactionerror(){ return transactionerror_signal; }
   /*! \brief  pos error signal */
   sigc::signal<void, int, void *> signal_poserror(){ return poserror_signal; }

   /*! \brief  hardware signal */
   sigc::signal<void, int, void *> signal_hw(){ return hw_signal; }
   /*! \brief  hardware error signal */
   sigc::signal<void, int, void *> signal_hwerror(){ return hwerror_signal; }
   
   /*! \brief Emit interface for plug-ins.
       \param emittype The type of signal to emit.
       \param msg The emit message.
       \param data Optional extra data.
       \return 0 if no error.
   */
   int Emit(BhMfEmitType emittype, int msg, void *data);
   // mf exposed general interfaces
   /*! \brief The general set option. 
       \param option The option to set */
   int  SetOption(BhMfConfig option, void *data);
   /*! \brief The general get option. */
   int  GetOption(BhMfConfig option, void *data);
   
   // mf exposed functional interfaces
   /*! \brief The general command interface.
       \param cmd The command.
       \param data Optional extra data.
       \return 0 if no error.
   */
   int  Command(BhMfCommand cmd, void *data);
   int  Command(BhMfCommand cmd, void *data, void *);
   /*! \brief The general system info interface.
       \param type The type of info.
       \param subtype Optional sub type.
       \param data pointer to be filled.
       \return 0 if no error.
   */
   int  GetSystemInfo(BhMfSystemInfoType type, int subtype, void *data);
   /*! \brief The general transaction info interface.
       \param type The type of info.
       \param transid The transaction id.
       \param data pointer to be filled.
       \return 0 if no error.
   */
   int  GetTransInfo(BhMfTransInfoType type, int transid, void *data);
   int  TransCycle(bhpos::CTransItems& transaction);
   int  HandlesKeyEvent(GdkEventKey *ev);
   /*! \brief Key event interface.
       \param ev The key event.
       \return non-zero if engine consumes key event.
   */
   int  KeyEvent(GdkEventKey *ev);
   int  HandlesDeviceEvent(bhpos::BhDevType type, int source, int size, void *data);
   int  Post(BhPosHdr *, bool);
   
   void HardwareMsg(bhpos::HardwareType, int, void *);
   void HardwareError(bhpos::HardwareType, int, void *);
};
/* @} */

}
/* @} */

#endif   // _MFPOSENGINE_H_
