/*
mfplugin.h
Copyright (C)2005 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
*/
#ifndef _MFPLUGIN_H_
#define _MFPLUGIN_H_

#include <glibmm/ustring.h>

#include <list>
#include <algorithm>

namespace bhmfengine{

class CMfPosEngine;

namespace plugin{
	
/**
 * @defgroup plugin Plug-in Modules
 * @{
 *
 * @ingroup mfengine
 * @{
 */

/*! \brief PluginException Class. */
class PluginException
{           
public:
   enum ErrorCode{ 
      seMemory, seBadProtocol, seServiceByName, 
      seBadHostName, seNoIPAddress, seDNSError,
      seMutexLock, seParameter, seOption,
      seNoProtocolSupport, seKernelMemory, seNoDescriptors,
      sePermissionDenied, seInvalidProtocol,
      seBadDescriptor, seBadPointer, seAlreadyConnected,
      seConnectRefused, seConnectTimeOut, seNetUnreachable,
      seAddrInUse, seInProgress, seAlreadyConnecting,
      seIncorrectAddrFamily, seBrdCastNotEnabled, seAlreadyBound,
      seAddressProtected, seCantListen, seNotStreamSock,
      seNoPendingConnections, seFirewall, seNotConnected,
      seWouldBlock, seIntseupted, seMessageSizeTooBig, 
      seNotBound, seOptionNotSupported,
      seWhoCares, seTotalWanker, seUnknown };

   PluginException(ErrorCode type);
   PluginException(const PluginException&);
   ~PluginException();

   operator const ErrorCode() { return type; }

private:
   ErrorCode     type;
   Glib::ustring errorstring;

   void SetString(const Glib::ustring errString);
};

typedef int  (*Initialize_t)(void *);
typedef int  (*CleanUp_t)();
typedef int  (*GetNumMsgHandlers_t)();
typedef int  (*GetMsgHandler_t)(int i);
typedef int  (*Handler_t)(unsigned int msg_type, int len, void *data, int flags);
typedef bool (*Lock_t)();
typedef bool (*Release_t)();

/*! \brief CMfPlugIn Class. 
   General Plugin base for plugin to inherited from.
*/
class CMfPlugIn{
protected:
   Glib::ustring fname;
   void	       *mod_handle;
   CMfPosEngine *mfeng;

	std::list<unsigned int> MsgTypes;

	// standard functions
	int   (*Initialize)(void *);
	int   (*CleanUp)();
	int   (*GetNumMsgHandlers)();
	int   (*GetMsgHandler)(int i);
	int   (*Handler)(unsigned int msg_type, int len, void *data, int flags);
	bool  (*Lock)();
	bool  (*Release)();

	int	GetHandlers();

public:
	CMfPlugIn();
	CMfPlugIn(CMfPlugIn const&);
	CMfPlugIn(CMfPosEngine *, Glib::ustring name, void *);
	~CMfPlugIn();
	void operator=(CMfPlugIn const&);
	bool operator==(CMfPlugIn const&);
	bool operator<(CMfPlugIn const&);
	
	void Close();
	int  Load(CMfPosEngine *, Glib::ustring, void *);
	int  Unload();
	bool CanHandle(unsigned int m);
	int  Process(unsigned int msg_type, int len, void *data, int flags);
};

/*! \brief CMfPlugInManager Class. 
   The manager for handling plugins.
*/
class CMfPlugInManager{
protected:
	std::list<CMfPlugIn>	Module;
	
public:
	CMfPlugInManager();
	CMfPlugInManager(CMfPlugInManager const&);
	CMfPlugInManager(CMfPosEngine *, char *dir, void *arg);
	~CMfPlugInManager();
	void operator=(CMfPlugInManager const&);
	bool operator==(CMfPlugInManager const&);
	bool operator<(CMfPlugInManager const&);
	
	bool ScanDirectory(CMfPosEngine *, char *dir, void *arg);
	void Close();

	int	HandleMsg(unsigned int msg_type, int len, void *data, int flags);
	bool  HasModule(char *modname);
	void *GetModuleFunction(char *modname, char *modfunc);

   // issued from mf engine
   int  Command(int, void *);
   void NetworkMessage();
   int  GetStatus();
};

/* }@ 
	}@ */

}

}

#endif	//	_MFPLUGIN_H_
