/*
connect.cpp - connect functions
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>

#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
#include "cmfengine.h"

namespace bhmfengine {

int CMfPosEngine::Connect()
{
   BhPosHdr	        hdr;
   BhPosConnect     c;
   
   memset(&hdr, 0, sizeof(hdr));
   memset(&c, 0, sizeof(c));
   hdr.hdr.type=BHPOS_TYPE_SYSTEM;
   hdr.hdr.cmd=BHMSG_SYSTEM_CONNECTION;
   hdr.hdr.sessionID=0;

   hdr.hdr.subcmd=BHMSG_SYSTEM_CONNECT;
   network->GetMac(c.mac);
   memcpy(hdr.data, &c, sizeof(BhPosConnect));		
   hdr.hdr.size=sizeof(BhPosConnect);
		
   hdr.hdr.Ex=0;
   hdr.hdr.flags=0;

   network->Send(&hdr, false, false, 0);
   return 0;
}

int CMfPosEngine::GetPrinterSettings()
{
   BhPosHdr hdr;
   int      c=0;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_PRINTER;
   hdr.hdr.size=0;
   
   switch(status){
   case MF_STATUS_PRNSETUP:
      c=1;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SETUP;
      break;
   case MF_STATUS_PRNINIT:
      c=2;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_INIT;
      break;
   case MF_STATUS_PRNPREPRINT:
      c=3;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_PREPRINT;
      break;
   case MF_STATUS_PRNHEADER:
      c=4;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_HDR;
      break;
   case MF_STATUS_PRNITEM:
      c=5;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_ITEM;
      break;
   case MF_STATUS_PRNTRANSSUM:
      c=6;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_TRANSSUM;
      break;
   case MF_STATUS_PRNPAYSUM:
      c=7;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_PAYSUM;
      break;
   case MF_STATUS_PRNFOOTER:
      c=8;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_FTR;
      break;
   case MF_STATUS_PRNPOSTPRINT:
      c=9;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_POSTPRINT;
      break;
   case MF_STATUS_JRNPREPRINT:
      c=10;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_JRNPREPRINT;
   	break;
   case MF_STATUS_JRNITEM:
      c=11;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_JRNITEM;
   	break;
   case MF_STATUS_JRNPOSTPRINT:
      c=12;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_JRNPOSTPRINT;
   	break;
   case MF_STATUS_LBLPREPRINT:
      c=13;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_LBLPREPRINT;
   	break;
   case MF_STATUS_LBLITEM:
      c=14;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_LBLITEM;
   	break;
   case MF_STATUS_LBLPOSTPRINT:
      c=15;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_LBLPOSTPRINT;
   	break;
   case MF_STATUS_CHKPREPRINT:
      c=16;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CHKPREPRINT;
   	break;
   case MF_STATUS_CHKITEM:
      c=17;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CHKITEM;
   	break;
   case MF_STATUS_CHKPOSTPRINT:
      c=18;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CHKPOSTPRINT;
   	break;   	
   case MF_STATUS_SLIPPREPRINT:
      c=19;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SLIPPREPRINT;
   	break;
   case MF_STATUS_SLIPHEADER:
      c=20;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SLIPHEADER;
   	break;
   case MF_STATUS_SLIPITEM:
      c=21;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SLIPITEM;
   	break;
   case MF_STATUS_SLIPFOOTER:
      c=22;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SLIPFOOTER;
   	break;
   case MF_STATUS_SLIPPOSTPRINT:
      c=23;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_SLIPPOSTPRINT;
   	break;   	
   case MF_STATUS_ENDPREPRINT:
      c=24;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_ENDPREPRINT;
   	break;
   case MF_STATUS_ENDITEM:
      c=25;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_ENDITEM;
   	break;
   case MF_STATUS_ENDPOSTPRINT:
      c=26;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_ENDPOSTPRINT;
   	break;
   case MF_STATUS_MICRPREREAD:
      c=27;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_MICRPREREAD;
   	break;
   case MF_STATUS_MICRREAD:
      c=28;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_MICRREAD;
   	break;
   case MF_STATUS_MICRSTATUS:
      c=29;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_MICRSTATUS;
   	break;
   case MF_STATUS_MICRPOSTREAD:
      c=30;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_MICRPOSTREAD;
   	break;
   case MF_STATUS_CUSTDPRESALE:
      c=31;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CUSTDISPPRESALE;
   	break;
   case MF_STATUS_CUSTDPOSTSALE:
      c=32;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CUSTDISPPOSTSALE;
   	break;
   case MF_STATUS_CUSTDIDLE:
      c=33;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &c);
      hdr.hdr.subcmd=BHMSG_POST_PRINTER_CUSTDISPIDLE;
   	break;
   }
   Post(&hdr, true);

   return 0;
}

int CMfPosEngine::GetPrefixList()
{
   BhPosHdr msg;
   status=MF_STATUS_PREFIXLIST;
   msg.hdr.type=BHPOS_TYPE_POST;
   msg.hdr.cmd=BHMSG_POST_LIST;
   msg.hdr.subcmd=BHMSG_POST_LIST_PREFIX;
   msg.hdr.size=0;
   Post(&msg, false);
   
   return 0;
}

int CMfPosEngine::GetPriceList()
{
   if(connection_info.numpriceitems!=0){
      status=MF_STATUS_PRICELIST;
      BhPosHdr msg;
      msg.hdr.type=BHPOS_TYPE_POST;
      msg.hdr.cmd=BHMSG_POST_LIST;
      msg.hdr.subcmd=BHMSG_POST_LIST_PRICE;
      msg.hdr.size=0;
      Post(&msg, false);
   }
   else{
      system_signal.emit(BHMF_EVENT_PRICELIST, NULL);
   }
   return 0;
}

int CMfPosEngine::GetTaxList()
{
   if(connection_info.numtaxitems!=0){
      BhPosHdr msg;
      status=MF_STATUS_TAXLIST;
      msg.hdr.type=BHPOS_TYPE_POST;
      msg.hdr.cmd=BHMSG_POST_LIST;
      msg.hdr.subcmd=BHMSG_POST_LIST_TAX;
      msg.hdr.size=0;
      Post(&msg, false);
      return 0;
   }
   else{
      system_signal.emit(BHMF_EVENT_TAXLIST, NULL);
   }
   return 0;
}

int CMfPosEngine::GetPayList()
{
   //if(connection_info.numpayitems!=0){
      BhPosHdr msg;
      status=MF_STATUS_PAYLIST;
      msg.hdr.type=BHPOS_TYPE_POST;
      msg.hdr.cmd=BHMSG_POST_LIST;
      msg.hdr.subcmd=BHMSG_POST_LIST_PAYTYPES;
      msg.hdr.size=0;
      Post(&msg, false);
      return 0;
   //}
   //else{
   //   system_signal.emit(BHMF_CMD_PAYLIST, NULL);
   //}
   return 0;
}

int CMfPosEngine::GetMenuHeadings()
{
   BhPosHdr msg;
   
   status=MF_STATUS_MENUHEADINGS;
   msg.hdr.type=BHPOS_TYPE_MENU;
   msg.hdr.cmd=BHMSG_MENU_HEADING;
   msg.hdr.subcmd=BHMSG_MENU_HEADING_GET;
   msg.hdr.size=0;
   Post(&msg, false);
   
   return 0;
}

int CMfPosEngine::GetMenuItems(int m)
{
   BhPosHdr           msg;
   BhMenuItemRequest *mir=(BhMenuItemRequest *) msg.data;
   
   status=MF_STATUS_MENUITEMS;
   msg.hdr.type=BHPOS_TYPE_MENU;
   msg.hdr.cmd=BHMSG_MENU_ITEM;
   msg.hdr.subcmd=BHMSG_MENU_ITEM_GET;
   mir->menuid=m;
   msg.hdr.size=sizeof(BhMenuItemRequest);
   Post(&msg, false);
   
   return 0;
}

int CMfPosEngine::Disconnect()
{
   BhPosHdr msg;
   
   status=MF_STATUS_DISCONNECTING;

   msg.hdr.type=BHPOS_TYPE_SYSTEM;
	msg.hdr.cmd=BHMSG_SYSTEM_CONNECTION;
	msg.hdr.subcmd=BHMSG_SYSTEM_DISCONNECT;
	msg.hdr.size=0;
   
   Post(&msg, false);
   
   return 0;
}

}
