/*
cpost.cpp - a point of sale client class
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>
#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
//#include "ckeymap.h"
#include "cmfengine.h"

namespace bhmfengine {

int CMfPosEngine::Configure()
{  
   Glib::ustring     c;
   unsigned short    p;
   bool              b;
   int               target;
   SDev              sdev;
   bhpos::CBhDevice *dev;
   
   bhpos::CBarcode  *barcode;
   bhpos::CPrinter  *printer;
   bhpos::CCashD    *cashdrawer;
   bhpos::CCustDisp *custdisp;
   bhpos::CMagswipe *magswipe;
   bhpos::CScales   *scales;
   
   bhpos::BarcodeDeviceType    dBcscanner;
   bhpos::CashdrawerDeviceType dCashd;
   bhpos::CustdispDeviceType   dCustdisp;
   bhpos::PrinterDeviceType    dPrinter;
   bhpos::MagSwipeDeviceType   dMagswipe;
   bhpos::ScalesDeviceType     dScales;
   
   config.GetOption(MFENG_CONFIG_BCSCANNER_DEVTYPE, &dBcscanner);
   switch(dBcscanner){
   case bhpos::BCDEV_TYPE_NONE:
      break;
   
	case bhpos::BCDEV_TYPE_KEYBOARD:
      break;
   
   case bhpos::BCDEV_TYPE_SERIAL:
      if(hardware.Add(bhpos::BHTYPE_BARCODE, bhpos::BHDEV_TYPE_SERIAL)){
         hwerror_signal.emit(BHMF_EVENT_HWERROR_BCADD, NULL);
      }
      dev=barcode=(bhpos::CBarcode *)hardware.GetDevice(bhpos::BHTYPE_BARCODE);
      config.GetOption(MFENG_CONFIG_BCSCANNER_DEVICE, &c);
      if(c.empty()){
         if(barcode->Open(c.data(), bhpos::BHDEV_NOTIFY_READ)){
            hwerror_signal.emit(BHMF_EVENT_HWERROR_BCOPEN, NULL);
         }
      }
      else{
         hwerror_signal.emit(BHMF_EVENT_HWERROR_BCDEVICE, NULL);
      }
      if(barcode){
         config.GetOption(MFENG_CONFIG_BCSCANNER_SSPEED, &target);
         dev->SetParameter(bhpos::BHDEV_PARAM_SERIALSPEED, &target);
         config.GetOption(MFENG_CONFIG_BCSCANNER_SBITS, &target);
         dev->SetParameter(bhpos::BHDEV_PARAM_SERIALBITS, &target);
         config.GetOption(MFENG_CONFIG_BCSCANNER_SSBITS, &target);
         dev->SetParameter(bhpos::BHDEV_PARAM_SERIALSTOPBITS, &target);
         config.GetOption(MFENG_CONFIG_BCSCANNER_SPARITY, &target);
         dev->SetParameter(bhpos::BHDEV_PARAM_SERIALPARITY, &target);
         //config.GetOption(MFENG_CONFIG_BCSCANNER_SSENTINEL, c);
         //barcode->SetParameter(bhpos::BCDEV_PARAM_SSENTINEL, c);
         //config.GetOption(MFENG_CONFIG_BCSCANNER_ESENTINEL, c);
         //barcode->SetParameter(bhpos::BCDEV_PARAM_ESENTINEL, c);
      }
      break;
   
   case bhpos::BCDEV_TYPE_PORTABLE:
      break;
   }
   if(dBcscanner!=bhpos::BCDEV_TYPE_NONE || dBcscanner!=bhpos::BCDEV_TYPE_KEYBOARD && barcode){
      //hardware.signal_read().connect(sigc::mem_fun(*this, &splash::EngMsg));
      //hw_signal.emit(HW_DEV_BCDONE, "Barcode Scanner Opened");
   }
   else{
   }
   /*
   sentinelstart=STX
   sentinelend=ETX
   targetdevice=/dev/ttyS0
   targetspeed=19200
   targetbits=8
   targetparity=NONE
   targetstopbits=1
   scannertype=symbol
   */   
   
   config.GetOption(MFENG_CONFIG_CASHDRAWER_DEVTYPE, &dCashd);
   switch(dCashd){
   case bhpos::CDDEV_TYPE_NONE:
      break;
   case bhpos::CDDEV_TYPE_SERIAL:
      hardware.Add(bhpos::BHTYPE_CASHDRAWER, bhpos::BHDEV_TYPE_SERIAL);
      dev=cashdrawer=(bhpos::CCashD *)hardware.GetDevice(bhpos::BHTYPE_CASHDRAWER);
      break;
   case bhpos::CDDEV_TYPE_SMARTSERIAL:
      hardware.Add(bhpos::BHTYPE_CASHDRAWER, bhpos::BHDEV_TYPE_SERIAL);
      dev=cashdrawer=(bhpos::CCashD *)hardware.GetDevice(bhpos::BHTYPE_CASHDRAWER);
      break;
   case bhpos::CDDEV_TYPE_PRINTER:
      break;
   }
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      break;
   case bhpos::CPDEV_TYPE_PRINTER:
      break;
   case bhpos::CPDEV_TYPE_SERIAL:
      hardware.Add(bhpos::BHTYPE_CUSTDISP, bhpos::BHDEV_TYPE_SERIAL);
      dev=custdisp=(bhpos::CCustDisp *)hardware.GetDevice(bhpos::BHTYPE_CUSTDISP);
      break;
   }
   
   config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
   switch(dPrinter){
   case bhpos::PRNDEV_TYPE_NONE:
      break;
   case bhpos::PRNDEV_TYPE_SERIAL:
      if(!(hardware.Add(bhpos::BHTYPE_PRINTER, bhpos::BHDEV_TYPE_SERIAL))){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTERADD, NULL);
         printf("failed to add printer\n");
      }
      dev=hardware.GetDevice(bhpos::BHTYPE_PRINTER);
      printer=dynamic_cast<bhpos::CPrinter *>(dev);
      if(!printer){
         printf("failed to cast printer\n");
      }
      config.GetOption(MFENG_CONFIG_PRINTER_DEVICE, &c);
      
      config.GetOption(MFENG_CONFIG_PRINTER_SSPEED, &target);
      printer->SetParameter(bhpos::BHDEV_PARAM_SERIALSPEED, &target);
      
      config.GetOption(MFENG_CONFIG_PRINTER_SBITS, &target);
      printer->SetParameter(bhpos::BHDEV_PARAM_SERIALBITS, &target);
      
      config.GetOption(MFENG_CONFIG_PRINTER_SPARITY, &target);
      printer->SetParameter(bhpos::BHDEV_PARAM_SERIALSTOPBITS, &target);
      
      config.GetOption(MFENG_CONFIG_PRINTER_SSBITS, &target);
      printer->SetParameter(bhpos::BHDEV_PARAM_SERIALPARITY, &target);
      
      printer->Open(c, bhpos::BHDEV_NOTIFY_READ);
      if(!printer->PrinterExists()){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTEROPEN, NULL);
         printf("Cant' find printer\n");
      }
      break;
      
	case bhpos::PRNDEV_TYPE_PARALLEL:
      if(!(hardware.Add(bhpos::BHTYPE_PRINTER, bhpos::BHDEV_TYPE_PARALLEL))){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTERADD, NULL);
         printf("failed to add printer\n");
      }
      dev=hardware.GetDevice(bhpos::BHTYPE_PRINTER);
      printer=dynamic_cast<bhpos::CPrinter *>(dev);
      if(!printer){
         printf("failed to cast printer\n");
      }
      config.GetOption(MFENG_CONFIG_PRINTER_DEVICE, &c);
      printer->Open(c, bhpos::BHDEV_NOTIFY_READ);
      if(!printer->PrinterExists()){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTEROPEN, NULL);
         printf("Cant' find printer\n");
      }
      break;
      
   case bhpos::PRNDEV_TYPE_USB:
      if(!(hardware.Add(bhpos::BHTYPE_PRINTER, bhpos::BHDEV_TYPE_USB))){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTERADD, NULL);
         printf("failed to add printer\n");
      }
      dev=hardware.GetDevice(bhpos::BHTYPE_PRINTER);
      printer=dynamic_cast<bhpos::CPrinter *>(dev);
      if(!printer){
         printf("failed to cast printer\n");
      }
      config.GetOption(MFENG_CONFIG_PRINTER_DEVICE, &c);
      printer->Open(c, bhpos::BHDEV_NOTIFY_READ);
      if(!printer->PrinterExists()){
         hwerror_signal(BHMF_EVENT_HWERROR_PRINTEROPEN, NULL);
         printf("Cant' find printer\n");
      }
      break;   
   
   case bhpos::PRNDEV_TYPE_NETWORK:
      break;
   }
   if(dPrinter!=bhpos::PRNDEV_TYPE_NONE && printer){
      b=true;
      config.GetOption(MFENG_CONFIG_PRINTER_RECCHARLINE, &target);
      printer->SetParam(bhpos::PRN_PARAM_RECCHARLINE, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_RECFONT, &target);
      printer->SetParam(bhpos::PRN_PARAM_RECFONT, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_JRNCHARLINE, &target);
      printer->SetParam(bhpos::PRN_PARAM_JRNCHARLINE, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_JRNFONT, &target);
      printer->SetParam(bhpos::PRN_PARAM_JRNFONT, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_SLIPCHARLINE, &target);
      printer->SetParam(bhpos::PRN_PARAM_SLIPCHARLINE, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_SLIPFONT, &target);
      printer->SetParam(bhpos::PRN_PARAM_SLIPFONT, &target);
      config.GetOption(MFENG_CONFIG_PRINTER_NVRAMSIZE, &target);
	   if(target!=0){
	      printer->SetParam(bhpos::PRN_PARAM_NVRAM, &b);
         printer->SetParam(bhpos::PRN_PARAM_NVRAMSIZE, &target);
      }
      
      config.GetOption(MFENG_CONFIG_PRINTER_BARCODE, &b);
      printer->SetParam(bhpos::PRN_PARAM_BARCODE, &b);
      
      config.GetOption(MFENG_CONFIG_PRINTER_PAGEMODE, &b);
      printer->SetParam(bhpos::PRN_PARAM_PAGEMODE, &b);
      
      config.GetOption(MFENG_CONFIG_PRINTER_AUTOCUSTDISP, &b);
      printer->SetParam(bhpos::PRN_PARAM_AUTOPOLE, &b);
      
      config.GetOption(MFENG_CONFIG_PRINTER_AUTOCUTTER, &b);
      printer->SetParam(bhpos::PRN_PARAM_AUTOCUTTER, &b);
      b=true;
      if(dCustdisp==bhpos::CPDEV_TYPE_PRINTER){
         printer->SetParam(bhpos::PRN_PARAM_CUSTDISP, &b);
         /*
         printer->SetParam(bhpos::PRN_PARAM_CUSTDISPTYPE, &custdisptype);*/
         config.GetOption(MFENG_CONFIG_CUSTDISP_LINEX, &target);
         printer->SetParam(bhpos::PRN_PARAM_CUSTDISPX, &target);
         config.GetOption(MFENG_CONFIG_CUSTDISP_LINEY, &target);
         printer->SetParam(bhpos::PRN_PARAM_CUSTDISPY, &target);
         
         // switch cursor off
         printer->Print(bhpos::PRNCMD_CDISPCMD, 2, NULL);
      }
      /*  
      if(dMicr==bhpos::MICR_TYPE_PRINTER){
         printer->SetParam(bhpos::PRN_PARAM_MICR, &b);
      }
      */
      
      if(print_strings.init){
         printer->Print(bhpos::PRNCMD_INTIALIZE, print_strings.init_size, print_strings.init);
      }
      config.GetOption(MFENG_CONFIG_PRINTER_CHARSET, &target);
      if(target!=0){
         printer->SetParam(bhpos::PRN_PARAM_CHARSET, &target);
      }
      
      if(dCashd==bhpos::CDDEV_TYPE_PRINTER){
         printer->SetParam(bhpos::PRN_PARAM_CASHDRAWER, &b);
         config.GetOption(MFENG_CONFIG_CASHDRAWER_FIREONBOOT, &b);
         if(b){
            printer->Print(bhpos::PRNCMD_CASHD, 0, NULL);
         }
      }
      config.GetOption(MFENG_CONFIG_PRINTER_DEFAULTCHK, &target);
      if(target!=0){
         int prn_xres, prn_yres;
         config.GetOption(MFENG_CONFIG_PRINTER_XRES, &prn_xres);
         config.GetOption(MFENG_CONFIG_PRINTER_YRES, &prn_yres);
         printer->SetParam(bhpos::PRN_PARAM_DEFXRES, &prn_xres);
         printer->SetParam(bhpos::PRN_PARAM_DEFYRES, &prn_yres);
         if(target <= DEFAULT_NUM_CHK){
            bhpos::ChequeLayout cl;
            memcpy(&cl, &bhpos::BhStdCheque[target-1], sizeof(bhpos::ChequeLayout));
            cl.x_dpi=prn_xres;
         	cl.y_dpi=prn_yres;
            printer->ChequeLayoutToUnit(&cl, bhpos::CHK_LAYOUT_MM);
         }
      }
   }
   
   // ME: do this for long time hardware calls...
   //hwthread = Glib::Thread::create(SigC::slot_class(*this, &CMfPosEngine::hwthreadfunc), true);

   hardware.signal_hw().connect(sigc::mem_fun(*this, &CMfPosEngine::HardwareMsg));
   hardware.signal_hwerror().connect(sigc::mem_fun(*this, &CMfPosEngine::HardwareError));
   system_signal.emit(BHMF_EVENT_HARDWARE, NULL);
   return 0;
}

void CMfPosEngine::HardwareMsg(bhpos::HardwareType type, int mtype, void *data)
{
   bool b;
   int  x=1;

   switch(type){
   case bhpos::BHTYPE_BARCODE:
      break;
   
   case bhpos::BHTYPE_PRINTER:
      switch(mtype){
      case bhpos::PRN_STATUSEVENT_SLIPSELECTED:
         break;
	   case bhpos::PRN_STATUSEVENT_SLIPLOADED:
         break;
      case bhpos::PRN_STATUSEVENT_SLIPCANCELLED:
         break;
      case bhpos::PRN_STATUSEVENT_SLIPREMOVED:
         break;
      case bhpos::PRN_STATUSEVENT_CHEQUESELECTED:
         break;
	   case bhpos::PRN_STATUSEVENT_CHEQUELOADED:
         hw_signal.emit(BHMF_EVENT_HW_CHEQUELOAD, NULL);
         break;
      case bhpos::PRN_STATUSEVENT_CHEQUECANCELLED:
         hw_signal.emit(BHMF_EVENT_HW_CHEQUECANCELED, NULL);
         break;
      case bhpos::PRN_STATUSEVENT_CHEQUEREMOVED:
         break;
	   case bhpos::PRN_STATUSEVENT_MICRSELECTED:
	   case bhpos::PRN_STATUSEVENT_MICRLOADED:
      case bhpos::PRN_STATUSEVENT_MICRREAD:
      case bhpos::PRN_STATUSEVENT_MICRCANCELLED:
      case bhpos::PRN_STATUSEVENT_MICRREMOVED:
         break;
      
      case bhpos::PRN_STATUSEVENT_CASHDRAWEROPEN:
         config.GetOption(MFENG_CONFIG_CASHDRAWER_BEEPONOPEN, &b);
         if(b){
            Command(bhmfengine::BHMF_CMD_SYSTEM_BEEP, &x);
         }
         hw_signal.emit(BHMF_EVENT_HW_CASHDRAWEROPEN, NULL);
         break;
      
      case bhpos::PRN_STATUSEVENT_CASHDRAWERCLOSED:
         // trans end, no sale, cd adjustment, logout drawer change
         if(status==MF_STATUS_NOSALE){
            status=MF_STATUS_IDLE;
         }
         else if(status==MF_STATUS_PAYIN){
            status=MF_STATUS_IDLE;
         }
         else if(status==MF_STATUS_PAYOUT){
            status=MF_STATUS_IDLE;
         }
         else if(status==MF_STATUS_PAYADJUSTMENT){
            status=MF_STATUS_IDLE;
         }
         else if(status==MF_STATUS_CDOPEN){
            // trans ended...
            config.GetOption(MFENG_CONFIG_LOGOUTEACHTRANS, &b);
            if(b){
               status=MF_STATUS_PASSIVELOGOUT;
            }
            else{
               status=MF_STATUS_IDLE;
            }
            //transaction_signal.emit(BHMF_CMD_TRANS_END, &current_transaction);
         }
         else if(status==MF_STATUS_ACK){
            if(tcstatus!=MF_STATUS_TRANSEND){
               // wait on trans end...
               //sigc::slot<bool> timer_slot = sigc::mem_fun(*this, &CMfPosEngine::TransTimer);
               //Glib::signal_timeout().connect(timer_slot, 100);
            }
            else{
               // clean up
            }
         }
         hw_signal.emit(BHMF_EVENT_HW_CASHDRAWERCLOSED, NULL);
         bcdOpen=false;
         break;
      }
      break;
      
   case bhpos::BHTYPE_CASHDRAWER:
   case bhpos::BHTYPE_CUSTDISP:
   case bhpos::BHTYPE_PORTABLE:
   case bhpos::BHTYPE_MAGSWIPE:
   case bhpos::BHTYPE_SCALES:
      break;
   }
}

void CMfPosEngine::HardwareError(bhpos::HardwareType type, int etype, void *data)
{
   switch(type){
   case bhpos::BHTYPE_BARCODE:
   case bhpos::BHTYPE_PRINTER:
   case bhpos::BHTYPE_CASHDRAWER:
   case bhpos::BHTYPE_CUSTDISP:
   case bhpos::BHTYPE_PORTABLE:
   case bhpos::BHTYPE_MAGSWIPE:
   case bhpos::BHTYPE_SCALES:
      break;
   }
}

void CMfPosEngine::Print(bhpos::BhPrintType type, void *data, int len)
{
   bhpos::PrinterDeviceType  dPrinter;
   
   config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
   if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return;
   bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
   if(!printer) return;
   
   switch(type){
   case bhpos::PRNCMD_PREPRINT:
      if(print_strings.preprint_size!=0) printer->Print(bhpos::PRNCMD_PREPRINT, print_strings.preprint_size, print_strings.preprint);
      return;
   //case bhpos::PRNCMD_TRANSSUMMARY:
      //return;
   case bhpos::PRNCMD_FOOTER:
      if(print_strings.footer_size!=0) printer->Print(bhpos::PRNCMD_FOOTER, print_strings.footer_size, print_strings.footer);
      return;
   //case bhpos::PRNCMD_POSTPRINT:
      //if(print_strings.postprint_size!=0) printer->Print(bhpos::PRNCMD_POSTPRINT, print_strings.postprint_size, print_strings.postprint);
      //return;
   case bhpos::PRNCMD_CDISPPRESALE:
      if(print_strings.custdisp_presale_size!=0) printer->Print(bhpos::PRNCMD_CDISPPRESALE, print_strings.custdisp_presale_size, print_strings.custdisp_presale);
      return;
   case bhpos::PRNCMD_CDISPPOSTSALE:
      if(print_strings.custdisp_postsale_size!=0) printer->Print(bhpos::PRNCMD_CDISPPOSTSALE, print_strings.custdisp_postsale_size, print_strings.custdisp_postsale);
      return;
   case bhpos::PRNCMD_CDISPIDLE:
      if(print_strings.custdisp_idle_size!=0) printer->Print(bhpos::PRNCMD_CDISPPOSTSALE, print_strings.custdisp_idle_size, print_strings.custdisp_idle);
      return;
      
   case bhpos::PRNCMD_CHQ_LOADTIMER:
      printer->Print(bhpos::PRNCMD_CHQ_LOADTIMER, 0, NULL);
      return;
   
   case bhpos::PRNCMD_CHQ_PRINT:
   {
      bhpos::ChequePrint chprn;
      tm		*Time;
		long	 CurrentLocalTime;
		time_t CurrentTime;
				
		CurrentLocalTime = time(&CurrentTime);
		Time = localtime(&CurrentLocalTime);
				
		sprintf(chprn.date, "%02d/%02d", Time->tm_mday, Time->tm_mon+1);
      sprintf(chprn.year, "%02d", Time->tm_year);
      
      if(print_strings.chk_item!=0){
	      strncpy(chprn.payto, print_strings.chk_item, 64);
	      strncpy(chprn.MajAmount, connection_info.currency_majname, 31);
	      strncpy(chprn.MinAmount, connection_info.currency_minname, 31);
	      g_print("\n%s - %s : %s %s %s\n", chprn.date,chprn.year,chprn.payto,chprn.MajAmount,chprn.MinAmount);
	      chprn.amount=CurrPayAuth.GetAmount();
	      chprn.endAccNum[0]=0;
      }
      printer->Print(bhpos::PRNCMD_CHQ_PRINT, 0, &chprn);
   }
      return;

   }
   printer->Print(type, len, data);
}            

int CMfPosEngine::CustomerDisplayTimer()
{
   // revert display to number of items, and running total
   static int revert=0;
   double a;
   int    n;
   char l1buff[21];
   char l2buff[21];
   char tbuff[21];
   int x;
   bool b;
   
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   bhpos::CPrinter *printer;
   
   if(!bCustDispRevertTimer) return 0;
      
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return 1;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
   }
      break;
   }
   
   if(cdisplaytype==0){ // sale item
      memset(l1buff, 0x20, 20);
      memset(l2buff, 0x20, 20);
      l1buff[20]=0;
      l2buff[20]=0;
      strncpy(l1buff, custdispSItem.GetRecText().data(), 20);
      sprintf(tbuff, "x %d", custdispSItem.GetQuantity());
      config.GetOption(MFENG_CONFIG_TAXINCLUSIVE, &b);
      const float bob=custdispSItem.GetTotal(b);
      memcpy(l2buff, tbuff, strlen(tbuff));
      sprintf(tbuff, "%0.2f", bob);
      x=20-strlen(tbuff);
      memcpy(&l2buff[x], tbuff, strlen(tbuff));
      printer->Print(bhpos::PRNCMD_CDISPLINE1, 1, l1buff);
      printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
      cdisplaytype=1;
   }
   else if(cdisplaytype==1){  // transinfo
      GetTransInfo(BHTRANSINFO_CURRENTNUMITEMS, 0, &n);
      GetTransInfo(BHTRANSINFO_CURRENTTOTAL, 0, &a);
   
      memset(l1buff, 0x20, 20);
      memset(l2buff, 0x20, 20);
      l1buff[20]=0;
      l2buff[20]=0;
      memcpy(l1buff, "Num Items : ", 12);
      sprintf(tbuff, "%d", n);
      x=20-strlen(tbuff);
      memcpy(&l1buff[x], tbuff, strlen(tbuff));
   
      memcpy(l2buff, "Total : ", 8);
      sprintf(tbuff, "%0.2f", a);
      x=20-strlen(tbuff);
      memcpy(&l2buff[x], tbuff, strlen(tbuff));

      printer->Print(bhpos::PRNCMD_CDISPLINE1, 1, l1buff);
      printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
      cdisplaytype=0;
   }
   else if(cdisplaytype==2){  // postmsg -> idle timer
      if(revert==2){
         revert=0;
         Print(bhpos::PRNCMD_CDISPIDLE, NULL, 0);
         return 0; // kill timer
      }
      revert++;
   }
   return 1;
}

void CMfPosEngine::CustomerDisplaySetTimer(bool bOn)
{
   int val;
   
   if(!bOn){
      cdisptimer_slot.disconnect();
      bCustDispRevertTimer=false;
      return;
   }
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_REVERTTIMER, &val);
   val=2000;
   if(val!=0){
      if(bCustDispRevertTimer){
         cdisptimer_slot.disconnect();
      }
      sigc::slot<bool> slot = sigc::mem_fun(*this, &CMfPosEngine::CustomerDisplayTimer);
      cdisptimer_slot = Glib::signal_timeout().connect(slot, val);
      bCustDispRevertTimer=true;
      return;
   }
   bCustDispRevertTimer=false;
}

void CMfPosEngine::CustomerDisplayClear()
{
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      break;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return;
      printer->Print(bhpos::PRNCMD_CDISPCLEAR, 0, (void *) "");
   }
      break;
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
}

void CMfPosEngine::CustomerDisplayCmd(int type)
{
   double a;
   int    n;
   char l1buff[21];
   char l2buff[21];
   char tbuff[21];
   int x;
   bool b;
   
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   
   memset(l1buff, 0x20, 20);
   memset(l2buff, 0x20, 20);
   l1buff[20]=0;
   l2buff[20]=0;
   
   CustomerDisplaySetTimer(false);
   CustomerDisplayClear();
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return;
      switch(type){
      case 1:
         //GetTransInfo(BHTRANSINFO_CURRENTNUMITEMS, 0, &n);
         GetTransInfo(BHTRANSINFO_CURRENTTOTAL, 0, &a);
         memcpy(l1buff, "Total : ", 8);
         sprintf(tbuff, "%0.2f", a);
         x=20-strlen(tbuff);
         memcpy(&l1buff[x], tbuff, strlen(tbuff));
         printer->Print(bhpos::PRNCMD_CDISPLINE1, 0, l1buff);
         break;
      
      case 2:
         GetTransInfo(BHTRANSINFO_CURRENTTOTAL, 0, &a);
         memcpy(l1buff, "Total : ", 8);
         sprintf(tbuff, "%0.2f", a);
         x=20-strlen(tbuff);
         memcpy(&l1buff[x], tbuff, strlen(tbuff));
         printer->Print(bhpos::PRNCMD_CDISPLINE1, 0, l1buff);
         
         GetTransInfo(BHTRANSINFO_CURRENTTOTALTENDERED, 0, &a);
         memcpy(l2buff, "Tender : ", 9);
         sprintf(tbuff, "%0.2f", a);
         x=20-strlen(tbuff);
         memcpy(&l2buff[x], tbuff, strlen(tbuff));
         printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
         break;
      
      case 3:  // display time
         printer->Print(bhpos::PRNCMD_CDISPCMD, 1, NULL);
         break;
      
      case 4:  // cursor off
         printer->Print(bhpos::PRNCMD_CDISPCMD, 2, NULL);
         break;
      }
   }
      break;
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
}

int CMfPosEngine::CustomerDisplay(char *msg)
{
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return 1;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
      printer->Print(bhpos::PRNCMD_CDISP, 1, msg);
   }
      break;
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
}

int CMfPosEngine::CustomerDisplay(bhpos::CSaleItem& si)
{
   char l1buff[21];
   char l2buff[21];
   char tbuff[21];
   int x;
   bool b;
   config.GetOption(MFENG_CONFIG_TAXINCLUSIVE, &b);
   const double bob=si.GetTotal(b);
      
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   
   memset(l1buff, 0x20, 20);
   memset(l2buff, 0x20, 20);
   l1buff[20]=0;
   l2buff[20]=0;
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return 1;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
      
      strncpy(l1buff, si.GetRecText().data(), 20);
      sprintf(tbuff, "x %d", si.GetQuantity());
      memcpy(l2buff, tbuff, strlen(tbuff));
      sprintf(tbuff, "%0.2f", bob);
      x=20-strlen(tbuff);
      memcpy(&l2buff[x], tbuff, strlen(tbuff));
      printer->Print(bhpos::PRNCMD_CDISPLINE1, 1, l1buff);
      printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
      custdispSItem=si;
      cdisplaytype=1;
      //if(!bCustDispRevertTimer){
         CustomerDisplaySetTimer(true);
      //}
   }
      break;
   
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
   return 0;
}


int CMfPosEngine::CustomerDisplay(bhpos::CPayItem& pi)
{
   char l1buff[21];
   char l2buff[21];
   char tbuff[21];
   int  x;
   bool b;
   double y;
   const double bob=pi.GetAmount();
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   Glib::ustring buff;
   
   memset(l1buff, 0x20, 20);
   memset(l2buff, 0x20, 20);
   l1buff[20]=0;
   l2buff[20]=0;
   
   bhpos::BhPayType pt=pi.GetID();
   
   switch(pt){
	case bhpos::PTYPE_UNKNOWN: strcpy(tbuff, ""); break;
	case bhpos::PTYPE_CASH:   strcpy(tbuff, "Cash :"); break;
	case bhpos::PTYPE_CREDIT: strcpy(tbuff, "Credit :"); break;
	case bhpos::PTYPE_CHEQUE: strcpy(tbuff, "Cheque :"); break;
	case bhpos::PTYPE_SMARTC: strcpy(tbuff, "Smart :"); break;
	case bhpos::PTYPE_CREDITRETURN:  strcpy(tbuff, "Credit Return :"); break;
   case bhpos::PTYPE_CREDITSLIP:  strcpy(tbuff,"Credit Slip :"); break;
	case bhpos::PTYPE_GIFTVOUCHER:  strcpy(tbuff, "Gift Voucher :"); break;
	case bhpos::PTYPE_TPVOUCHER:  strcpy(tbuff, "External Voucher :"); break;
	case bhpos::PTYPE_INTCOUPON: strcpy(tbuff, "Internal Coupon :"); break;
   case bhpos::PTYPE_TPCOUPON:  strcpy(tbuff, "External Coupon :"); break;
	case bhpos::PTYPE_LUNCHSTAMP:  strcpy(tbuff, "Lunch Stamps :"); break;
   case bhpos::PTYPE_INHOUSECHARGE: strcpy(tbuff, "In-house Charge :"); break;
   case bhpos::PTYPE_STORECARD:  strcpy(tbuff, "Store Card :"); break;
	case bhpos::PTYPE_OTHER:  strcpy(tbuff, "Other :"); break;
	case bhpos::PTYPE_CREDIT_CB: strcpy(tbuff, "Credit (Cash Back) :"); break;
	case bhpos::PTYPE_CHEQUE_CB: strcpy(tbuff, "Cheque (Cash Back) :"); break;
	case bhpos::PTYPE_SMARTC_CB: strcpy(tbuff, "Smart (Cash Back) :"); break;
	case bhpos::PTYPE_TOKEN_CB:  strcpy(tbuff, "Token (Cash Back) :"); break;
	case bhpos::PTYPE_VOUCHER_CB:  strcpy(tbuff, "Voucher (Cash Back) :"); break;
	case bhpos::PTYPE_COUPON_CB:  strcpy(tbuff, "Coupon (Cash Back) :"); break;
	case bhpos::PTYPE_OTHER_CB:  strcpy(tbuff, "Other (Cash Back) :"); break;
	}
   
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return 1;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
         
      memcpy(l2buff, tbuff, strlen(tbuff));
      sprintf(tbuff, "%0.2f", bob);
      x=20-strlen(tbuff);
      memcpy(&l2buff[x], tbuff, strlen(tbuff));
      if(pt==bhpos::PTYPE_CASH){
         printer->Print(bhpos::PRNCMD_CDISPLINE1, 0, l2buff);
         memset(l2buff, 0x20, 20);
         l2buff[20]=0;
         strcpy(tbuff, "Change :");
         memcpy(l2buff, tbuff, strlen(tbuff));
         GetTransInfo(BHTRANSINFO_CURRENTCHANGE, 0, &y);
         sprintf(tbuff, "%0.2f", y);
         x=20-strlen(tbuff);
         memcpy(&l2buff[x], tbuff, strlen(tbuff));
         printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
      }
      else{
         printer->Print(bhpos::PRNCMD_CDISPLINE2, 0, l2buff);
      }
   }
      break;
   
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
   return 0;
}


int CMfPosEngine::CustomerDisplay(int line, bool bClear, char *msg)
{
   bhpos::CustdispDeviceType dCustdisp;
   bhpos::PrinterDeviceType  dPrinter;
   int x=0;
   
   if(bClear) x=1;
   config.GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &dCustdisp);
   switch(dCustdisp){
   case bhpos::CPDEV_TYPE_NONE:
      return 1;
   case bhpos::CPDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
      char lbuff[21];
      
      memset(lbuff, 0x20, 20);
      lbuff[20]=0;
      
      strncpy(lbuff, msg, 20);
      if(line==1){
         printer->Print(bhpos::PRNCMD_CDISPLINE1, x, lbuff);
      }
      else if(line==2){
         printer->Print(bhpos::PRNCMD_CDISPLINE2, x, lbuff);
      }
   }
      break;
   case bhpos::CPDEV_TYPE_SERIAL:
      break;
   }
}

int CMfPosEngine::CashDrawerTimer()
{
   bhpos::CashdrawerDeviceType dCashd;
   bhpos::PrinterDeviceType    dPrinter;
   config.GetOption(MFENG_CONFIG_CASHDRAWER_DEVTYPE, &dCashd);
   switch(dCashd){
   case bhpos::CDDEV_TYPE_NONE:
      bcdOpen=false;
      return 1;
   
   case bhpos::CDDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
      printer->Print(bhpos::PRNCMD_CASHDTIMER, 0, NULL);
   }
      break;
   case bhpos::CDDEV_TYPE_SERIAL:
      break;
   
   case bhpos::CDDEV_TYPE_SMARTSERIAL:
      break;
   }
   
   return 0;
}

int CMfPosEngine::CashDrawer(int type)
{
   bhpos::CashdrawerDeviceType dCashd;
   bhpos::PrinterDeviceType    dPrinter;
   config.GetOption(MFENG_CONFIG_CASHDRAWER_DEVTYPE, &dCashd);
   switch(dCashd){
   case bhpos::CDDEV_TYPE_NONE:
   {
      // just return a "closed event"
      //timespec ts1={0,100000000}, ts2;
      //nanosleep(&ts1, &ts2);
      //hw_signal.emit(BHMF_EVENT_HW_CASHDRAWERCLOSED, NULL);
      bcdOpen=false;
   }
      return 1;
   
   case bhpos::CDDEV_TYPE_PRINTER:
   {
      config.GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &dPrinter);
      if(dPrinter==bhpos::PRNDEV_TYPE_NONE) return 2;
      bhpos::CPrinter *printer=dynamic_cast<bhpos::CPrinter *>(hardware.GetDevice(bhpos::BHTYPE_PRINTER));
      if(!printer) return 3;
      printer->Print(bhpos::PRNCMD_CASHD, type, NULL);
      bcdOpen=true;
   }
      break;
   case bhpos::CDDEV_TYPE_SERIAL:
      break;
   
   case bhpos::CDDEV_TYPE_SMARTSERIAL:
      break;
   }
   
   return 0;
}

/*

void CMfPosEngine::JustifyLine(int flags, char *s, char *e)
{
	char linebuffer[256];
	
	char nl=0x0A;
	memset(linebuffer, 0x20, custdispx);
	
	if(flags==0){
		memcpy(linebuffer, s, strlen(s));
		memcpy((linebuffer + (custdispx - strlen(e))), e, strlen(e));
		Output(reccharline, linebuffer);
	}
	else if(flags==1){
		memcpy((linebuffer + (custdispx - strlen(e))), e, strlen(e));
		Output(reccharline, linebuffer);
	}	
		
	Output(1, &nl);
}
*/
/*
int CMfPosEngine::InputStream(GdkEventAny *event)
{
   if(stream_manager){
      ret=stream_manager.Handler(event);
   }
   return ret;
}

int CMfPosEngine::DeviceStream(int source, GdkInputCondition condition)
{
   return 0;
}

int CMfPosEngine::EventHandler(BhEventAny *bhevent)
{
}
*/

}
