/*
pluginexception.h
Copyright (C)2005 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
*/
#include "cmfplugin.h"

namespace bhmfengine{
   
namespace plugin{

PluginException::PluginException(ErrorCode code = seUnknown) : type(code)
{
   switch(type){
   case seBadProtocol:          SetString("Specified Protocol in invalid\n"); break;
   case seServiceByName:        SetString("Specified Service Name is invalid\n"); break;
   case seBadHostName:          SetString("Specified Host Name is invalid\n"); break;
   case seNoIPAddress:          SetString("No IP Address for this host\n"); break;
   case seDNSError:             SetString("DNS Error\n"); break;
   case seMutexLock:            SetString("Mutex Failed to lock\n"); break;
   case seParameter:            SetString("Bad Parameter Passed\n"); break;
   case seNoProtocolSupport:    SetString("Protocol Type not Supported\n"); break;
   case seNoDescriptors:        SetString("No Descriptors left for Socket\n"); break;
   case sePermissionDenied:     SetString("Permission to create Socket denied\n"); break;
   case seInvalidProtocol:      SetString("Protocol unknown or not available\n"); break;
   case seBadDescriptor:        SetString("Bad Socket Descriptor\n"); break;
   case seBadPointer:           SetString("Pointer is a bastard\n"); break;
   case seAlreadyConnected:     SetString("The socket is already connected\n"); break;
   case seConnectRefused:       SetString("Server refused the Connection\n"); break;
   case seConnectTimeOut:       SetString("Timeout while attempting connection\n"); break;
   case seNetUnreachable:       SetString("The Network is unreachable\n"); break;
   case seAddrInUse:            SetString("The provided Address is already in use\n"); break;
   case seInProgress:           SetString("Socket is Non-Blocking, operation currently in progress\n"); break;
   case seAlreadyConnecting:    SetString("Socket is Non-Blocking, previous connect still in progress\n"); break;
   case seIncorrectAddrFamily:  SetString("Incorrect Address Family\n"); break;
   case seBrdCastNotEnabled:    SetString("Broadcast flag was not enabled\n"); break;
   case seAlreadyBound:         SetString("The socket is already bound to an address\n"); break;
   case seAddressProtected:     SetString("Permission to Access the specified Port Number is denied\n"); break;
   case seCantListen:           SetString("Listen Operation not Supported on this Socket\n"); break;
   case seNotStreamSock:        SetString("Cannot Accept Connection - not a TCP Socket\n"); break;
   case seNoPendingConnections: SetString("Socket is Non-Blocking, no pending connections\n"); break;
   case seFirewall:             SetString("Firewall rules forbid connection\n"); break;
   case seNotConnected:         SetString("Illegal operation on an unconnected Socket\n"); break;
   case seWouldBlock:           SetString("Socket is Non-Blocking, requested operation would block\n"); break;
   case seIntseupted:           SetString("The send/receive was intseupted by a signal\n"); break;
   case seMessageSizeTooBig:    SetString("Message too Big for atomic transmission\n"); break;
   case seNotBound:             SetString("Local Socket is not bound to an address/port number\n"); break;
   case seOptionNotSupported:   SetString("This options is unsupported at the specified Code Level\n"); break;
   case seTotalWanker:          SetString("You're a total Wanker...\n"); break;
   case seUnknown:				  SetString("An unknown error occured\n"); break;
   case seOption:					  SetString("Bad option\n"); break;
	case seKernelMemory:			  SetString("Kernel Memory Error\n"); break;
	case seMemory:					  SetString("Memory Error\n"); break;
	case seWhoCares:       	     SetString("An error has occurred... Who Cares (c) MS\n"); break;       
	
   }
}

PluginException::PluginException(const PluginException& pex)
{
    type=pex.type;
    SetString(pex.errorstring);
}


PluginException::~PluginException()
{
}

void PluginException::SetString(const Glib::ustring s)
{
   errorstring=s;
}

}

}
