/*
postmsg.cc 
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#include "config.h"
#include <cstdio>
#include <cstring>

#include "cmfengine.h"

namespace bhmfengine {

void CMfPosEngine::NetworkMessage(int size, void *data)
{
   BhPosHdr *hdr=(BhPosHdr *) data;
   
   switch(hdr->hdr.type){
   case BHPOS_TYPE_NULL: break;
   case BHPOS_TYPE_SYSTEM: SystemMessage(hdr); break;
   case BHPOS_TYPE_POST: PostMessage(hdr); break;
   case BHPOS_TYPE_CUSTOM:
   case BHPOS_TYPE_ERROR:
   case BHPOS_TYPE_SERVICE:
   case BHPOS_TYPE_INFO:
   case BHPOS_TYPE_SCALES:
   case BHPOS_TYPE_RPRINTER:
   case BHPOS_TYPE_ENTRY:
   case BHPOS_TYPE_ATTENDANCE:
   case BHPOS_TYPE_SYNCH:
   case BHPOS_TYPE_SERVERSIDE:
      break;
   }
}

/***************************************************************
 *
 * Post Messages
 *
 ***************************************************************/

int CMfPosEngine::PostMessage(BhPosHdr *hdr)
{
   int  i;
   bool bm, bt;
   bhpos::BhTransactionType tctype;
	
   BhPrinterMsg *pm;
   BhPLUreply    plur;
   
   config.GetOption(MFENG_CONFIG_MODAL, &bm);
   config.GetOption(MFENG_CONFIG_LOGOUTEACHTRANS, &bt);
   //g_print("status: %d\n", status);
   switch(status){
   case MF_STATUS_TRANSID:
   case MF_STATUS_TRANSIDONLY:
   {
      BhTransactionNew trn;
      if(hdr->hdr.Ex==BHPOS_REFUSE){
      }
      else{
         memcpy(&trn, hdr->data, sizeof(trn));
         current_transaction=trn.transID;
         current_index=transstack.Add(trn.transID, userID, userName);
         transstack.SetType(trn.transID, bhpos::BHTRANSACTION_TYPE_TRANSACTION);
         //transstack.SetUserName(trn.transID, userName);
         //g_print("setting user:%s\n", userName);
         transaction_signal.emit(BHMF_EVENT_TRANS_ID, &trn.transID);
         if(status!=MF_STATUS_TRANSIDONLY){
            transaction_signal.emit(BHMF_EVENT_SITEM_LOOKUP, NULL);
            SaleItemLookup();
            return 0;
         }
         else{
            //if(!bLookup){
               CustomerDisplay(CurrPluLookup);
               transstack.AddItem(current_transaction, &CurrPluLookup, true);
               transaction_signal.emit(BHMF_EVENT_SITEM_UPDATE, NULL);
               //status=MF_STATUS_IDLE;
               bKeyDisabled=false;
            //}
         }
         status=MF_STATUS_IDLE;
         mode=MF_MODE_SALE;
      }
   }
      break;
   
   case MF_STATUS_TRANSIDADD:
   {
      BhTransactionNew trn;
      if(hdr->hdr.Ex==BHPOS_REFUSE){
      }
      else{
         memcpy(&trn, hdr->data, sizeof(trn));
         current_transaction=trn.transID;
         current_index=transstack.Add(trn.transID);
         transaction_signal.emit(BHMF_EVENT_TRANS_IDADD, &trn.transID);
         transstack.AddItem(current_transaction, &CurrPluLookup, true);
         status=MF_STATUS_IDLE;
         bKeyDisabled=false;
         mode=MF_MODE_SALE;
      }
   }
      break;
      
   case MF_STATUS_PLULOOKUP:
      memset(&plur, 0, sizeof(plur));
      GetOption(MFENG_CONFIG_PRINTER_AUTOCUSTDISP, &bt);
      if(hdr->hdr.Ex==BHPOS_REFUSE){
         GetOption(MFENG_CONFIG_UPLUPOLICY, &i);
         if(i==0){
            CurrPluLookup.SetType(bhpos::BHSI_SITYPE_PRICE);
            status=MF_STATUS_PRICEENTRY;
         }
         if(i==1){
            CurrPluLookup.SetType(bhpos::BHSI_SITYPE_DESCPRICE);
            status=MF_STATUS_DESCRIPTIONENTRY;
         }
         if(i==2){
            CurrPluLookup.SetType(bhpos::BHSI_SITYPE_UPLU);
            status=MF_STATUS_UNKNOWNPLU;
         }
         CurrPluLookup.SetDispText("Unknown Item");
         CurrPluLookup.SetRecText("Item");
         CurrPluLookup.SetPrice(0); 
         CurrPluLookup.SetFlags(0);
         if(bt){
            //revert disp to total
         }
         transactionerror_signal.emit(BHMF_EVENT_SITEM_RETURN, (void *) &CurrPluLookup);
         
         bKeyDisabled=false;
         return 0;
      }
      else{
         memcpy(&plur, hdr->data, sizeof(plur));
         *(hdr->data + (hdr->hdr.size - sizeof(BhPLUreply)))=0;
         CurrPluLookup.SetDispText(plur.displayText);
         CurrPluLookup.SetRecText(plur.recuText);
         CurrPluLookup.SetTax(plur.tax);
         CurrPluLookup.SetDepositType((bhpos::BhItemDepositType) plur.deposittype);
         CurrPluLookup.SetDeposit(plur.deposit);
         CurrPluLookup.SetDiscount(plur.discount);
         CurrPluLookup.SetPrice(plur.unitPrice);
#ifdef _DEBUGGERY
         g_print("type:%d\n", CurrPluLookup.GetType());
#endif   
         CurrPluLookup.SetType(bhpos::BHSI_SITYPE_PLU);
         CurrPluLookup.SetFlags(plur.flags);
         /*
         if(plur.flags==ITEMFLAG_TYPE_MSG){
            CurrPluLookup.SetMsg(hdr->data + sizeof(plur));
         }*/
         CustomerDisplay(CurrPluLookup);
         transstack.AddItem(current_transaction, &CurrPluLookup, true);
         transaction_signal.emit(BHMF_EVENT_SITEM_RETURN, (void *) &CurrPluLookup);
         //status=MF_STATUS_IDLE;
         bKeyDisabled=false;
      }
      break;
/*      
   case MF_STATUS_EMPLU:
      memset(&plur, 0, sizeof(plur));
      
      if(hdr->hdr.Ex==BHPOS_REFUSE){
         PLUErr((int) *hdr->data);
         return 0;
      }
      else{
			BhTransEphemLookupReturn *er=(BhTransEphemLookupReturn *) hdr->data;
			strncpy(plur.displayText, er->rxnumber, 32);
   		strcpy(plur.recuText, "");
   		plur.tax=0.00;
   		plur.discount=0.00;
   		plur.unitPrice=er->price;
         PLURet(&plur, NULL);
		}
      break;
   
   case MF_STATUS_CLOCKINACK:
      if(trans){
   	   trans->SetFlag(BHTRANSFLAGTYPE_CLOCKIN, false);
   	}
   	status=MF_STATUS_IDLE;
      if(hdr->hdr.Ex==BHPOS_REFUSE){
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Unknown Employee"),				
			   "/usr/share/bhpos/pixmaps/icon_nodelivery.xpm",
   			_("This employee was not found."),
			   NULL,
   			NULL));   	   
      }
      break;
		
	case MF_STATUS_MICRCOMMIT:
		if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
         hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_MICRCOMMIT){
			if(hdr->hdr.Ex==BHPOS_REFUSE){
				trans->MicrDone(MICRSTATUS_COMMITFAIL);
			}
			else{
				trans->MicrDone(MICRSTATUS_OK);
			}
		}
		break;

   case MF_STATUS_ZTAPE:
		if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_SERVICE &&
         hdr->hdr.subcmd==BHMSG_POST_SERVICE_SUMMARY_ZTAPE){
			if(hdr->hdr.Ex==BHPOS_GRANT){
            if(dPrinter!=POST_PRINTER_NONE && printer){
               bhpos::HeaderPrint h;
               h.UserID=userID;
			      h.StationID=connection_info.PostID;
					tm		*ltm;
		         long	 lt;
			      time_t t;
				
			      lt = time(&t);
			      ltm = localtime(&lt);
			      strftime(h.datetime, sizeof(h.datetime), "%x", ltm);
			      char buff[8];
			      sprintf(buff, " - %02d:%02d", ltm->tm_hour, ltm->tm_min);
			      strcat(h.datetime, buff);
               
               printer->PrintZTape(&h, (BhServiceTransSum *) hdr->data);
					status=MF_STATUS_IDLE;
            	trans->SetStatus(TRANSSTATE_IDLE);
            }
         }
         else{
            status=MF_STATUS_IDLE;
            trans->SetStatus(TRANSSTATE_IDLE);
         }
		}
		break;
   
   case MF_STATUS_RETURNREASON:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         trans->ReturnReason((BhReturnReasonList *) hdr->data);
      }
      else{
         status=MF_STATUS_IDLE;
         trans->SetStatus(TRANSSTATE_IDLE);
      }
      break;
      
   case MF_STATUS_TRANSRETRIEVE:
		if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION){
      	TransRetreive(hdr);
		}
      break;
   */
   case MF_STATUS_PAYAUTH:
      if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
         hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_PAYAUTH){
         BhTransPayAuthReply *r=(BhTransPayAuthReply *) hdr->data;
         CurrPayAuth.SetFlags((bhpos::BhPayTypeFlag) r->flags);
         pos_signal.emit(BHMF_EVENT_PITEM_AUTHRETURN, (BhTransPayAuthReply *) hdr->data);
         //CreditDone((BhTransPayAuthReply *) hdr->data);
      }
      break;

   case MF_STATUS_ADJUSTMENT:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         switch(hdr->hdr.subcmd){
         case BHMSG_POST_ADJUST_CASHDRAWER:
            pos_signal.emit(BHMF_EVENT_SYSTEM_PAYADJUSTMENT, NULL);
            break;
         
         case BHMSG_POST_ADJUST_PAYIN:
            pos_signal.emit(BHMF_EVENT_SYSTEM_PAYIN, NULL);
            break;
         
         case BHMSG_POST_ADJUST_PAYOUT:
            pos_signal.emit(BHMF_EVENT_SYSTEM_PAYOUT, NULL);
            break;
         }
      }
      else{
         switch(hdr->hdr.subcmd){
         case BHMSG_POST_ADJUST_CASHDRAWER:
            poserror_signal.emit(BHMF_EVENT_SYSTEM_PAYADJUSTMENT, NULL);
            break;
         
         case BHMSG_POST_ADJUST_PAYIN:
            poserror_signal.emit(BHMF_EVENT_SYSTEM_PAYIN, NULL);
            break;
         
         case BHMSG_POST_ADJUST_PAYOUT:
            poserror_signal.emit(BHMF_EVENT_SYSTEM_PAYOUT, NULL);
            break;
         }
      }
      status=MF_STATUS_IDLE;
      break;
      
   case MF_STATUS_VOIDACK:
      if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
         hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_CANCEL && hdr->hdr.Ex==BHPOS_GRANT){
         transstack.Clear(current_transaction);
         if(transstack.NumTrans()==0){
            current_transaction=-1;
         }
         else {
            current_transaction=transstack.NumTrans()-1;
         }
         if(bt)
            status=MF_STATUS_PASSIVELOGOUT;
         else
            status=MF_STATUS_IDLE;
         
         transaction_signal.emit(BHMF_EVENT_TRANS_VOID, &current_transaction);
         CustomerDisplaySetTimer(false);
         Print(bhpos::PRNCMD_CDISPPRESALE, NULL, 0);
         return 0;
      }
      break;
      
   case MF_STATUS_ACK:
      tctype=transstack.GetType(current_transaction);
      //g_print("bonkersfish tctype :%d\n", tctype);
      if(tctype==bhpos::BHTRANSACTION_TYPE_TRANSACTION){
         if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
         	hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT){
         	if(hdr->hdr.Ex==BHPOS_GRANT){
         	   // an ack
       	  	   tcstatus=MF_STATUS_PLUCOMMIT;
         	}
         	else{
            	tcstatus=MF_STATUS_TRANSCOMMIT;
         	}
         	TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
         	hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PLU){
				if(hdr->hdr.Ex==BHPOS_GRANT){
 	         }
  	         else{
   	      }
    	      TransCommit();
     	   }
     	   else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_RETURNS_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         } 
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_RENTAL_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_LAYBY_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }
      	else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_UPLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
            else{
            }
	         TransCommit();
 	      }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PRICEPLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
            else{
            }
	         TransCommit();
 	      }
  	      else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
   	      hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PT){
    	      if(hdr->hdr.Ex==BHPOS_GRANT){
     	      }
      	   else{
            }
            TransCommit();
         }
	      else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_END){
  	         if(hdr->hdr.Ex==BHPOS_GRANT){
               transstack.Clear(current_transaction);
               if(transstack.NumTrans()==0){
                  current_transaction=-1;
               }
               else {
                  current_transaction=-1;
                  //current_transaction=transstack.NumTrans()-1;
               }
               //if(bcdOpen){
                  //g_print("\n\nDONKEY SHOES\n\n");
               //   status=MF_STATUS_CDOPEN;
               //}
               //else{
                  //if(bt)
                     //status=MF_STATUS_PASSIVELOGOUT;
                  //else
                     //status=MF_STATUS_IDLE;
                  
                  
               //}
               CurrPluLookup.Clear();
               custdispSItem.Clear();
               custdispPItem.Clear();
               CurrPayAuth.Clear();
               
               //CustomerDisplaySetTimer(false);
               //CustomerDisplayClear();
               //Print(bhpos::PRNCMD_CDISPPOSTSALE, NULL, 0);
               transaction_signal(BHMF_EVENT_TRANS_END, &current_transaction);               
    	         //tcstatus=BHTRANSACTION_TYPE_NONE;
      	   }
         }
      }
      else if(tctype==bhpos::BHTRANSACTION_TYPE_DELIVERY){
         if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT){
            if(hdr->hdr.Ex==BHPOS_GRANT){
               tcstatus=MF_STATUS_PLUCOMMIT;
            }
            else{
               tcstatus=MF_STATUS_TRANSCOMMIT;
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_RENTAL_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_LAYBY_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }
 	     else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_UPLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
            else{
            }
  	       TransCommit();
        }
        else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
           hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_END){
           if(hdr->hdr.Ex==BHPOS_GRANT){
              transstack.Clear(current_transaction);
               if(transstack.NumTrans()==0){
                  current_transaction=-1;
               }
               else {
                  current_transaction=transstack.NumTrans()-1;
               }
               if(bcdOpen){
                  status=MF_STATUS_CDOPEN;
               }
               else{
                  if(bt)
                     status=MF_STATUS_PASSIVELOGOUT;
                  else
                     status=MF_STATUS_IDLE;
                  transaction_signal(BHMF_CMD_TRANS_END, &current_transaction);
               }
               CustomerDisplaySetTimer(false);
               CustomerDisplayClear();
               Print(bhpos::PRNCMD_CDISPPOSTSALE, NULL, 0);
            }
         }
      }
      else if(tctype==bhpos::BHTRANSACTION_TYPE_EPHEMERAL){
         if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT){
            if(hdr->hdr.Ex==BHPOS_GRANT){
               tcstatus=MF_STATUS_PLUCOMMIT;
            }
            else{
               tcstatus=MF_STATUS_TRANSCOMMIT;
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }   
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
           hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_END){
           if(hdr->hdr.Ex==BHPOS_GRANT){
              transstack.Clear(current_transaction);
               if(transstack.NumTrans()==0){
                  current_transaction=-1;
               }
               else {
                  current_transaction=transstack.NumTrans()-1;
               }
               if(bt)
                  status=MF_STATUS_PASSIVELOGOUT;
               else
                  status=MF_STATUS_IDLE;
               transaction_signal(BHMF_CMD_TRANS_END, &current_transaction);
    	         //tcstatus=BHTRANSACTION_TYPE_NONE;
            }
         }
      }
      else if(tctype==bhpos::BHTRANSACTION_TYPE_RETURNS){
        if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
           hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT){
           if(hdr->hdr.Ex==BHPOS_GRANT){
 	          transstack.Clear(current_transaction);
               if(transstack.NumTrans()==0){
                  current_transaction=-1;
               }
               else {
                  current_transaction=transstack.NumTrans()-1;
               }
               if(bt)
                  status=MF_STATUS_PASSIVELOGOUT;
               else
                  status=MF_STATUS_IDLE;
               transaction_signal(BHMF_CMD_TRANS_END, &current_transaction);
   	         
    	         //tcstatus=BHTRANSACTION_TYPE_NONE;
            }
         }
      }
      else if(tctype==bhpos::BHTRANSACTION_TYPE_RETURNSPASSIVE){
         if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT){
            if(hdr->hdr.Ex==BHPOS_GRANT){
               tcstatus=MF_STATUS_PLUCOMMIT;
            }
            else{
               tcstatus=MF_STATUS_TRANSCOMMIT;
            }
            TransCommit();
         }
         else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_PLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
	         else{
            }
            TransCommit();
         }   
 	     else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
            hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_COMMIT_UPLU){
            if(hdr->hdr.Ex==BHPOS_GRANT){
            }
            else{
            }
  	         TransCommit();
        }
        else if(hdr->hdr.type==BHPOS_TYPE_POST && hdr->hdr.cmd==BHMSG_POST_TRANSACTION &&
           hdr->hdr.subcmd==BHMSG_POST_TRANSACTION_RETURNS_END){
           if(hdr->hdr.Ex==BHPOS_GRANT){
            transstack.Clear(current_transaction);
               if(transstack.NumTrans()==0){
                  current_transaction=-1;
               }
               else {
                  current_transaction=transstack.NumTrans()-1;
               }
               if(bt)
                  status=MF_STATUS_PASSIVELOGOUT;
               else
                  status=MF_STATUS_IDLE;
               transaction_signal(BHMF_CMD_TRANS_END, &current_transaction);
   	         
    	         //tcstatus=BHTRANSACTION_TYPE_NONE;
            }
         }
      }
      else if(tctype==bhpos::BHTRANSACTION_TYPE_RETRIEVE){
      }
      else{
      }
      break;

   case MF_STATUS_TRANSAUTH:
      break;

   case MF_STATUS_TRANS:
      break;

   case MF_STATUS_PRNSETUP:
      status=MF_STATUS_PRNINIT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.setup) delete print_strings.setup;
         print_strings.setup_size=pm->size;
         print_strings.setup = new char[pm->size];
         if(print_strings.setup) memcpy(print_strings.setup, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("setup string:%s\n", print_strings.setup);
#endif         
      }     
      GetPrinterSettings();
      break;

   case MF_STATUS_PRNINIT:
      status=MF_STATUS_PRNPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.init) delete print_strings.init;
         print_strings.init_size=pm->size;
         print_strings.init = new char[strlen(pm->data) + 1];
         if(print_strings.init) memcpy(print_strings.init, pm->data, pm->size);
#ifdef _DEBUGGERY
         g_print("prn init string:%s\n", print_strings.init);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNPREPRINT:
      status=MF_STATUS_PRNHEADER;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.preprint) delete print_strings.preprint;
         print_strings.preprint_size=pm->size;
         print_strings.preprint = new char[pm->size];
         if(print_strings.preprint) memcpy(print_strings.preprint, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn preprint string:%s\n", print_strings.preprint);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNHEADER:
      status=MF_STATUS_PRNITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.header) delete print_strings.header;
         print_strings.header_size=pm->size;
         print_strings.header = new char[pm->size];
         if(print_strings.header) memcpy(print_strings.header, pm->data, pm->size);
#ifdef _DEBUGGERY
         g_print("prn header string:%d\n", print_strings.header_size);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNITEM:
      status=MF_STATUS_PRNTRANSSUM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.item) delete print_strings.item;
         print_strings.postprint_size=pm->size;
         print_strings.postprint = new char[pm->size];
         if(print_strings.item) memcpy(print_strings.item, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn item string:%s\n", print_strings.item);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNTRANSSUM:
      status=MF_STATUS_PRNPAYSUM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.transsum) delete print_strings.transsum;
         print_strings.transsum_size=pm->size;
         print_strings.transsum = new char[pm->size];
         if(print_strings.transsum) memcpy(print_strings.transsum, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn transsum string:%s\n", print_strings.transsum);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNPAYSUM:
      status=MF_STATUS_PRNFOOTER;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.paysum) delete print_strings.paysum;
         print_strings.paysum_size=pm->size;
         print_strings.paysum = new char[pm->size];
         if(print_strings.paysum) memcpy(print_strings.paysum, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn paysum string:%s\n", print_strings.paysum);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNFOOTER:
      status=MF_STATUS_PRNPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.footer) delete print_strings.footer;
         print_strings.footer_size=pm->size;
         print_strings.footer = new char[pm->size];
         if(print_strings.footer) memcpy(print_strings.footer, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn footer string:%s\n", print_strings.footer);
#endif          
      }     
      GetPrinterSettings();
      break;
   case MF_STATUS_PRNPOSTPRINT:
      status=MF_STATUS_JRNPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.postprint) delete print_strings.postprint;
         print_strings.postprint_size=pm->size;
         print_strings.postprint = new char[pm->size];
         if(print_strings.postprint) memcpy(print_strings.postprint, pm->data, pm->size);
#ifdef _DEBUGGERY
         //g_print("prn post string:%d - %s\n", print_strings.postprint_size, print_strings.postprint);
#endif          
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_JRNPREPRINT:
      status=MF_STATUS_JRNITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.jrn_preprint) delete print_strings.jrn_preprint;
         print_strings.jrn_preprint_size=pm->size;
         print_strings.jrn_preprint = new char[pm->size];
         if(print_strings.jrn_preprint) memcpy(print_strings.jrn_preprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_JRNITEM:
      status=MF_STATUS_JRNPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.jrn_item) delete print_strings.jrn_item;
         print_strings.jrn_item_size=pm->size;
         print_strings.jrn_item = new char[pm->size];
         if(print_strings.jrn_item) memcpy(print_strings.jrn_item, pm->data, pm->size);
      }
      //g_print("jrn prn item print\n");
      GetPrinterSettings();
      break;
   case MF_STATUS_JRNPOSTPRINT:
      status=MF_STATUS_LBLPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.jrn_postprint) delete print_strings.jrn_postprint;
         print_strings.jrn_postprint_size=pm->size;
         print_strings.jrn_postprint = new char[pm->size];
         if(print_strings.jrn_postprint) memcpy(print_strings.jrn_postprint, pm->data, pm->size);
      }
      //g_print("jrn prn post print\n");
      GetPrinterSettings();
      break;
   case MF_STATUS_LBLPREPRINT:
      status=MF_STATUS_LBLITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.lbl_preprint) delete print_strings.lbl_preprint;
         print_strings.lbl_preprint_size=pm->size;
         print_strings.lbl_preprint = new char[pm->size];
         if(print_strings.lbl_preprint) memcpy(print_strings.lbl_preprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_LBLITEM:
      status=MF_STATUS_LBLPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.lbl_item) delete print_strings.lbl_preprint;
         print_strings.lbl_item_size=pm->size;
         print_strings.lbl_item = new char[pm->size];
         if(print_strings.lbl_item) memcpy(print_strings.lbl_item, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_LBLPOSTPRINT:
      status=MF_STATUS_CHKPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.lbl_postprint) delete print_strings.lbl_postprint;
         print_strings.lbl_postprint_size=pm->size;
         print_strings.lbl_postprint = new char[pm->size];
         if(print_strings.lbl_postprint) memcpy(print_strings.lbl_postprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CHKPREPRINT:
      status=MF_STATUS_CHKITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.chk_preprint) delete print_strings.chk_preprint;
         print_strings.chk_preprint_size=pm->size;
         print_strings.chk_preprint = new char[pm->size];
         if(print_strings.chk_preprint) memcpy(print_strings.chk_preprint, pm->data, pm->size);
         //g_print("chk pre prnt print\n");
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CHKITEM:
      status=MF_STATUS_CHKPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.chk_item) delete print_strings.chk_preprint;
         print_strings.chk_item_size=pm->size;
         print_strings.chk_item = new char[pm->size];
         if(print_strings.chk_item) memcpy(print_strings.chk_item, pm->data, pm->size);
         //g_print("chk item prn print\n");
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CHKPOSTPRINT:
      status=MF_STATUS_ENDPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.chk_postprint) delete print_strings.chk_postprint;
         print_strings.chk_postprint_size=pm->size;
         print_strings.chk_postprint = new char[pm->size];
         if(print_strings.chk_postprint) memcpy(print_strings.chk_postprint, pm->data, pm->size);
         //g_print("chk port prn print\n");
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_ENDPREPRINT:
      status=MF_STATUS_ENDITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.end_preprint) delete print_strings.end_preprint;
         print_strings.end_preprint_size=pm->size;
         print_strings.end_preprint = new char[pm->size];
         if(print_strings.end_preprint) memcpy(print_strings.end_preprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_ENDITEM:
      status=MF_STATUS_ENDPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.end_item) delete print_strings.end_preprint;
         print_strings.end_item_size=pm->size;
         print_strings.end_item = new char[pm->size];
         if(print_strings.end_item) memcpy(print_strings.end_item, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_ENDPOSTPRINT:
      status=MF_STATUS_SLIPPREPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.end_postprint) delete print_strings.end_postprint;
         print_strings.end_postprint_size=pm->size;
         print_strings.end_postprint = new char[pm->size];
         if(print_strings.end_postprint) memcpy(print_strings.end_postprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_SLIPPREPRINT:
      status=MF_STATUS_SLIPHEADER;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.slip_preprint) delete print_strings.slip_preprint;
         print_strings.slip_preprint_size=pm->size;
         print_strings.slip_preprint = new char[pm->size];
         if(print_strings.slip_preprint) memcpy(print_strings.slip_preprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_SLIPHEADER:
      status=MF_STATUS_SLIPITEM;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.slip_header) delete print_strings.slip_header;
         print_strings.slip_header_size=pm->size;
         print_strings.slip_header = new char[pm->size];
         if(print_strings.slip_header) memcpy(print_strings.slip_header, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_SLIPITEM:
      status=MF_STATUS_SLIPFOOTER;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.slip_item) delete print_strings.slip_item;
         print_strings.slip_item_size=pm->size;
         print_strings.slip_item = new char[pm->size];
         if(print_strings.slip_item) memcpy(print_strings.slip_item, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_SLIPFOOTER:
      status=MF_STATUS_SLIPPOSTPRINT;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.slip_footer) delete print_strings.slip_footer;
         print_strings.slip_footer_size=pm->size;
         print_strings.slip_footer = new char[pm->size];
         if(print_strings.slip_footer) memcpy(print_strings.slip_footer, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_SLIPPOSTPRINT:
      status=MF_STATUS_MICRPREREAD;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.slip_postprint) delete print_strings.slip_postprint;
         print_strings.slip_postprint_size=pm->size;
         print_strings.slip_postprint = new char[pm->size];
         if(print_strings.slip_postprint) memcpy(print_strings.slip_postprint, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_MICRPREREAD:
      status=MF_STATUS_MICRREAD;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.micr_preread) delete print_strings.micr_preread;
         print_strings.micr_preread_size=pm->size;
         print_strings.micr_preread = new char[pm->size];
         if(print_strings.micr_preread) memcpy(print_strings.micr_preread, pm->data, pm->size);
         g_print("micr pre prn print\n");
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_MICRREAD:
      status=MF_STATUS_MICRSTATUS;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.micr_read) delete print_strings.micr_read;
         print_strings.micr_read_size=pm->size;
         print_strings.micr_read = new char[pm->size];
         if(print_strings.micr_read) memcpy(print_strings.micr_read, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_MICRSTATUS:
      status=MF_STATUS_MICRPOSTREAD;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.micr_status) delete print_strings.micr_status;
         print_strings.micr_status_size=pm->size;
         print_strings.micr_status = new char[pm->size];
         if(print_strings.micr_status) memcpy(print_strings.micr_status, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_MICRPOSTREAD:
      status=MF_STATUS_CUSTDPRESALE;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.micr_postread) delete print_strings.micr_postread;
         print_strings.micr_postread_size=pm->size;
         print_strings.micr_postread = new char[pm->size];
         if(print_strings.micr_postread) memcpy(print_strings.micr_postread, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CUSTDPOSTSALE:
      status=MF_STATUS_CUSTDPOSTSALE;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.custdisp_postsale) delete print_strings.custdisp_postsale;
         print_strings.custdisp_postsale_size=pm->size;
         print_strings.custdisp_postsale = new char[pm->size];
         if(print_strings.custdisp_postsale) memcpy(print_strings.custdisp_postsale, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CUSTDIDLE:
      status=MF_STATUS_CUSTDIDLE;
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.custdisp_idle) delete print_strings.custdisp_idle;
         print_strings.custdisp_idle_size=pm->size;
         print_strings.custdisp_idle = new char[pm->size];
         if(print_strings.custdisp_idle) memcpy(print_strings.custdisp_idle, pm->data, pm->size);
      }
      GetPrinterSettings();
      break;
   case MF_STATUS_CUSTDPRESALE:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         pm=(BhPrinterMsg *) hdr->data;
         if(print_strings.custdisp_presale) delete print_strings.custdisp_presale;
         print_strings.custdisp_presale_size=pm->size;
         print_strings.custdisp_presale = new char[pm->size+1];
         if(print_strings.custdisp_presale){ 
            memcpy(print_strings.custdisp_presale, pm->data, pm->size);
            print_strings.custdisp_presale[pm->size]=0;
         }
      }
      i=0;
      system_signal.emit(BHMF_EVENT_PRINTERSETTINGS, &i);
      break;
      
   case MF_STATUS_PREFIXLIST:
      if(hdr->hdr.Ex==BHPOS_GRANT){
#ifdef _DEBUGGERY          
         g_print("global prefix list...\n");
#endif         
         memcpy(&bc_prefix, (GlobalPrefix*) hdr->data, sizeof(GlobalPrefix));
      }
      system_signal.emit(BHMF_EVENT_PREFIXLIST, NULL);
      break;
      
   case MF_STATUS_TAXLIST:
   {
      if(hdr->hdr.Ex==BHPOS_GRANT){
#ifdef _DEBUGGERY
         g_print("Setting tax list from server: %d\n", status);
#endif
         BhTaxList *tl=(BhTaxList *) hdr->data;
         BhTaxListItem *tli=tl->item;
         BhTaxListItem *tlt;
         taxlist.numitems=tl->numitems;
         taxlist.item=new BhTaxListItem[taxlist.numitems];
   
         if(taxlist.item==NULL) return 1;
   
         tlt=taxlist.item;
   
         for(int i=0; i < tl->numitems; i++){
   	      memcpy(tlt, tli, sizeof(BhTaxListItem));
         	tlt++;
   	      tli++;
      	}
      }
      system_signal.emit(BHMF_EVENT_TAXLIST, NULL);
   }
      break;
   
   case MF_STATUS_PRICELIST:
      if(hdr->hdr.Ex==BHPOS_GRANT){
#ifdef _DEBUGGERY
         g_print("Setting price list from server: %d\n", status);
#endif
         BhPriceList *pl=(BhPriceList *) hdr->data;
         
         BhPriceListItem *pli=pl->item;
         BhPriceListItem *tpl;
         pricelist.num_items=pl->numitems;
         pricelist.item=new BhPriceListItem[pricelist.num_items];
   
         if(pricelist.item==NULL) return 1;
   
         tpl=pricelist.item;
   
         for(int i=0; i < pl->numitems; i++){
            tpl->ID=pli->ID;
            tpl->AccountID=pli->AccountID;
            strcpy(tpl->name, pli->name);
            //g_print("sale label:%s", tpl->name);
         	tpl++;
   	      pli++;
         }
      }
      system_signal.emit(BHMF_EVENT_PRICELIST, NULL);
      break;
      
   case MF_STATUS_MENUHEADINGS:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         //SetPriceList((BhPriceList *) hdr->data);
#ifdef _DEBUGGERY
         g_print("Setting menu headings from server: %d\n", status);
#endif
         BhMenuHeadingList *mhl=(BhMenuHeadingList *) hdr->data;
         BhMenuHeading     *pmh=mhl->items;
         BhMenuHeading     *mh;
         
         menuheadings.numitems=mhl->num_items;
         menuheadings.item=new BhMenuHeading[menuheadings.numitems];
   
         if(menuheadings.item==NULL) return 1;
   
         mh=menuheadings.item;
   
         for(int i=0; i < mhl->num_items; i++){
            mh->id=pmh->id;
            strncpy(mh->name, pmh->name, 32);
            strncpy(mh->image, pmh->image, 64);
         	mh++;
   	      pmh++;
         }
      }
      system_signal.emit(BHMF_EVENT_MENUHEADINGS, NULL);
      break;
   
   case MF_STATUS_MENUITEMS:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         //SetPriceList((BhPriceList *) hdr->data);
#ifdef _DEBUGGERY
         g_print("Setting menu items server: %d\n", status);
#endif
         BhMenuItemList *mil=(BhMenuItemList *) hdr->data;
         BhMenuItem     *pmi=mil->items;
         BhMenuItem     *mi;
         
         menuitems.numitems=mil->num_items;
         menuitems.item=new BhMenuItem[menuitems.numitems];
   
         if(menuitems.item==NULL) return 1;
   
         mi=menuitems.item;
   
         for(int i=0; i < mil->num_items; i++){
            mi->id=pmi->id;
            strncpy(mi->name, pmi->name, 32);
            strncpy(mi->rectext, pmi->rectext, 32);
            strncpy(mi->quantity, pmi->quantity, 32);
            strncpy(mi->image, pmi->image, 64);
            mi->price=pmi->price;
            mi->tax=pmi->tax;
            mi->printerid=pmi->printerid;
         	mi++;
   	      pmi++;
         }
      }
      system_signal.emit(BHMF_EVENT_MENUITEMS, NULL);
      break;
   
   case MF_STATUS_PAYLIST:
      if(hdr->hdr.Ex==BHPOS_GRANT){
#ifdef _DEBUGGERY
         g_print("Setting pay list from server: %d\n", status);
#endif
         BhExPaytypeList *pl=(BhExPaytypeList *) hdr->data;
         
         BhExPaytype *pli=pl->item;
         BhExPaytype *tpl;
         paylist.numitems=pl->num_types;
         paylist.item=new BhExPaytype[paylist.numitems];
   
         if(paylist.item==NULL){
            return 1;
         }
   
         tpl=paylist.item;
   
         for(int i=0; i < pl->num_types; i++){
            tpl->ID=pli->ID;
            strcpy(tpl->name, pli->name);
            tpl->cashback=pli->cashback;
            tpl->limit=pli->limit;
         	tpl++;
   	      pli++;
         }
      }
      system_signal.emit(BHMF_EVENT_PAYLIST, NULL);
      break;

   case MF_STATUS_TRANSSUM:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         //trans->TransactionSummary((BhServiceTransSum *) hdr->data);
         
      }
      break;
   case MF_STATUS_USERSUM:
      if(hdr->hdr.Ex==BHPOS_GRANT){
         //trans->UserSummary((BhServiceUserSum *) hdr->data);
      }
      break;
      
   case MF_STATUS_CASHDACK:
      /*
      trans->SetStatus(TRANSSTATE_IDLE);
      status=MF_STATUS_IDLE;*/
      break;
   
   case MF_STATUS_CUSTDELIVERY:
   {
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
         trans->CustomerSummary((BhCustomerList *) hdr->data);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Unknown Customer"),				
				"/usr/share/bhpos/pixmaps/icon_nocustomer.xpm",
				_("This customer was not found..."),
				NULL,
				NULL));
			trans->SetFlag(BHTRANSFLAGTYPE_DELIVERY, false);
			trans->SetStatus(TRANSSTATE_IDLE);
      }*/
   }
      break;
      
   case MF_STATUS_CUSTRENTAL:
   {
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
         trans->CustomerSummary((BhCustomerList *) hdr->data);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Unknown Customer"),				
				"/usr/share/bhpos/pixmaps/icon_nocustomer.xpm",
				_("This customer was not found..."),
				NULL,
				NULL));
			trans->SetFlag(BHTRANSFLAGTYPE_RENTAL, false);
      }*/
   }
      break;
      
   case MF_STATUS_CUSTSUMMARY:
	{
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
			trans->CustomerSummary((BhCustomerList *) hdr->data);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Unknown Customer"),				
				"/usr/share/bhpos/pixmaps/icon_nocustomer.xpm",
				_("This customer was not found..."),
				NULL,
				NULL));
      }
      */
	}
      break;
   
   case MF_STATUS_CUSTLAYBY:
   {
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
			trans->CustomerSummary((BhCustomerList *) hdr->data);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_YESNO, bModal, _("Unknown Customer"),				
				"/usr/share/bhpos/pixmaps/icon_nocustomer.xpm",
				_("This customer was not found... New Layby?"),
				&newlayby_ok,
				&newlayby_cancel));		   
      }
      */
	}
      break;
   
   case MF_STATUS_DELIVERYLIST:
   {
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
			trans->DeliveryList((BhDeliveryList *) hdr->data);
      }
      else{
         if(*((int *) hdr->data) == 1){
            ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("No Deliveries"),				
			   	"/usr/share/bhpos/pixmaps/icon_nodelivery.xpm",
   				_("There are no outstanding deliveries"),
			   	NULL,
   				NULL));
			}
			else{
			   ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Unknown Employee"),				
			   	"/usr/share/bhpos/pixmaps/icon_nodelivery.xpm",
   				_("This employee was not found."),
			   	NULL,
   				NULL));
			}
			status=MF_STATUS_IDLE;
			trans->SetStatus(TRANSSTATE_IDLE);
      }
      */
	}
      break;
   
   case MF_STATUS_CUSTNEW:
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
         int cid=(int) hdr->data;
			trans->CustomerNew(true, cid);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Add Customer Failure"),				
				"/usr/share/bhpos/pixmaps/icon_nocustomer.xpm",
				_("Failed to add customer."),
				NULL,
				NULL));
      }*/
      break;
      
   case MF_STATUS_BLINDBALANCE:
   {
      /*
      if(hdr->hdr.Ex==BHPOS_GRANT){
			BhBlindBalanceReply     *r=(BhBlindBalanceReply *) hdr->data;
			trans->BlindBalance(r);
      }
      else{
         ct=manage(new confirm(BHCONFIRMBOX_OK, bModal, _("Cash Default Error"),				
				"/usr/share/bhpos/pixmaps/icon_badamount.xpm",
				_("No cash drawer defaults set."),
				NULL,
				NULL));
		   trans->SetStatus(TRANSSTATE_IDLE);
		   
      }*/
	}
      break;
   }
    return 0;
}

}
