/*
saleitem.cpp - a point of sale client class
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2004 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>

#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
#include "cmfengine.h"

namespace bhmfengine {

BhMfPluType CMfPosEngine::BarcodeType(bhpos::CSaleItem& si)
{
   char strplu[129];
   int  type=0;
   
   memcpy(strplu, si.GetPLU().data(), 2);
	strplu[2]=0;
	type=atoi(strplu);
   
	if(type==bc_prefix.weight_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for weight\n");
#endif		
		return BHPLU_TYPE_WEIGHT;
   }

   if(type==bc_prefix.quantity_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for quantity\n");
#endif		
		return BHPLU_TYPE_QUANTITY;
   }
   
   if(type==bc_prefix.interface_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for interface\n");
#endif				
		return BHPLU_TYPE_INTERFACE;
   }

   if(type==bc_prefix.price_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for price\n");
#endif				
		return BHPLU_TYPE_PRICE;
   }

   if(type==bc_prefix.token_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for token\n");
#endif				
		return BHPLU_TYPE_TOKEN;
   }

   if(type==bc_prefix.voucher_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for voucher\n");
#endif				
		return BHPLU_TYPE_VOUCHER;
   }

   if(type==bc_prefix.instore_prefix){
#ifdef _DEBUGGERY			 
      g_print("Plu: request for in-store\n");
#endif				
		return BHPLU_TYPE_INSTORE;
   }
	
#ifdef _DEBUGGERY 
   g_printf("Plu: request for standard lookup\n");
#endif
   
   return BHPLU_TYPE_ITEM;
}

int CMfPosEngine::SaleItemAdd(bhpos::CSaleItem& si, bool bl)
{
   BhPosHdr         hdr;
   BhPLUrequest     plureq;

   memset(&hdr, 0, sizeof(hdr));
   memset(&plureq, 0, sizeof(plureq));
   bLookup=bl;
   
   if(BarcodeType(si)!=BHPLU_TYPE_ITEM){
      return 0;
   }
   
   if(si==CurrPluLookup){
      //g_print("same item");
      CurrPluLookup.SetQuantity(CurrPluLookup.GetQuantity() + si.GetQuantity());
      transstack.Update(current_transaction, CurrPluLookup);
      transaction_signal.emit(BHMF_EVENT_SITEM_RETURN, (void *) &CurrPluLookup);
      //mutex.unlock();
      return 0;
   }
   else{
      if(current_transaction==-1){
         // request a new trans ID from the server
         CurrPluLookup=si;
         if(bl){
            status=MF_STATUS_TRANSID;
         }
         else{
            status=MF_STATUS_TRANSIDONLY;
         }
         hdr.hdr.type=BHPOS_TYPE_POST;
         hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
         hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_NEW;
         hdr.hdr.size=0;
         network->Send(&hdr, false, false, 0);
         //mutex.unlock();
         bKeyDisabled=true;
         bLookup=false;
         return 1;
      }
      else{
         if(!bl){
            transstack.AddItem(current_transaction, &si, true);
            transaction_signal.emit(BHMF_EVENT_SITEM_UPDATE, NULL);
            bKeyDisabled=false;
            bLookup=false;
            return 1;
         }
      }
      status=MF_STATUS_PLULOOKUP;
      CurrPluLookup=si;
      
      //if(bSessionTimeout){
      //   sessiontimer.disconnect();
      //   bSessionTimeout=false;
      //}
   
      plureq.transID=current_transaction;
      plureq.type=0;
      strcpy(plureq.PLU, CurrPluLookup.GetPLU().data());
      hdr.hdr.type=BHPOS_TYPE_POST;
      hdr.hdr.cmd=BHMSG_POST_PLU;
      hdr.hdr.subcmd=BHMSG_POST_PLU_LOOKUP;
      hdr.hdr.Ex=0;
      hdr.hdr.size=sizeof(plureq);
      memcpy(hdr.data, &plureq, sizeof(plureq));
      network->Send(&hdr, false, false, 0);
      bKeyDisabled=true;
   }
  
   return 0;
}

int CMfPosEngine::SaleItemAdd(float& p)
{
   BhPosHdr         hdr;

   memset(&hdr, 0, sizeof(hdr));
   
   CurrPluLookup.Clear();
   CurrPluLookup.SetPLU("------------");
   CurrPluLookup.SetDispText("Cash Item");
   CurrPluLookup.SetRecText("Cash Item");
   // erm...
   //CurrPluLookup.SetTax(plur.tax);
   CurrPluLookup.SetPrice(p);
   CurrPluLookup.SetType(bhpos::BHSI_SITYPE_PRICE);
   if(current_transaction==-1){
      status=MF_STATUS_TRANSIDADD;
      hdr.hdr.type=BHPOS_TYPE_POST;
      hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
      hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_NEW;
      hdr.hdr.size=0;
      network->Send(&hdr, false, false, 0);
      //mutex.unlock();
      bKeyDisabled=true;
      return 1;
   }
   else{
      transstack.AddItem(current_transaction, &CurrPluLookup, true);
      status=MF_STATUS_IDLE;
      bKeyDisabled=false;
   }
   return 0;
}

int CMfPosEngine::SaleItemLookup()
{
   BhPosHdr         hdr;
   BhPLUrequest     plureq;

   memset(&hdr, 0, sizeof(hdr));
   memset(&plureq, 0, sizeof(plureq));
   
   status=MF_STATUS_PLULOOKUP;
   plureq.transID=current_transaction;
   plureq.type=0;
   strcpy(plureq.PLU, CurrPluLookup.GetPLU().data());
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_PLU;
   hdr.hdr.subcmd=BHMSG_POST_PLU_LOOKUP;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(plureq);
   memcpy(hdr.data, &plureq, sizeof(plureq));
   network->Send(&hdr, false, false, 0);
   bKeyDisabled=true;
}

int CMfPosEngine::SaleItemLookup(bhpos::CSaleItem& si)
{
   BhPosHdr         hdr;
   BhPLUrequest     plureq;

   memset(&hdr, 0, sizeof(hdr));
   memset(&plureq, 0, sizeof(plureq));
   
   if(BarcodeType(si)!=BHPLU_TYPE_ITEM){
      return 0;
   }
   
   if(si==CurrPluLookup){
      CurrPluLookup.SetQuantity(CurrPluLookup.GetQuantity() + si.GetQuantity());
      transstack.Update(current_transaction, CurrPluLookup);
      //mutex.unlock();
      return 0;
   }
   else{
      status=MF_STATUS_PLULOOKUP;
      CurrPluLookup=si;
      
      //if(bSessionTimeout){
      //   sessiontimer.disconnect();
      //   bSessionTimeout=false;
      //}
   
      plureq.transID=current_transaction;
      plureq.type=0;
      strcpy(plureq.PLU, CurrPluLookup.GetPLU().data());
      hdr.hdr.type=BHPOS_TYPE_POST;
      hdr.hdr.cmd=BHMSG_POST_PLU;
      hdr.hdr.subcmd=BHMSG_POST_PLU_LOOKUP;
      hdr.hdr.Ex=0;
      hdr.hdr.size=sizeof(plureq);
      memcpy(hdr.data, &plureq, sizeof(plureq));
      network->Send(&hdr, false, false, 0);
      bKeyDisabled=true;
   }
  
   return 0;
}

int CMfPosEngine::SaleItemEdit(bhpos::CSaleItem& si)
{
}

int CMfPosEngine::SaleItemRemove(bhpos::CSaleItem& si)
{
}

int CMfPosEngine::SaleItemEdit(bhpos::CSaleItem& si, int n)
{
   return transstack.Update(current_transaction, bhpos::BHTRANSEDITTYPE_QTY, n, si);
}

int CMfPosEngine::SaleItemRemove(bhpos::CSaleItem& si, int n)
{
}

void CMfPosEngine::SaleItemRemoveByScan()
{
   status=MF_STATUS_SCANREMOVE;
}

}
