/*
setup.cpp - a point of sale client class
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2004 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>

#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
#include "cmfengine.h"

namespace bhmfengine {

void CMfPosEngine::Adjustment(int type, double amount)
{
   BhPosHdr hdr;
   
   memset(&hdr, 0, sizeof(hdr));
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_ADJUST;
   hdr.hdr.subcmd=type;
   hdr.hdr.Ex=0;
   
   switch(type){
   case BHMSG_POST_ADJUST_CASHDRAWER:
   {
      BhCashDrawerAdjust *da=(BhCashDrawerAdjust *) hdr.data;
      hdr.hdr.size=sizeof(BhCashDrawerAdjust);
      
   }
      break;
   
   case BHMSG_POST_ADJUST_PAYIN:
   {
      BhPayinAdjust *pi=(BhPayinAdjust *) hdr.data;
      
      hdr.hdr.size=sizeof(BhPayinAdjust);
      type=0;
      pi->amount=amount;
   }
      break;
   
   case BHMSG_POST_ADJUST_PAYOUT:
   {
      BhPayoutAdjust *po=(BhPayoutAdjust *) hdr.data;
      
      hdr.hdr.size=sizeof(BhPayoutAdjust);
      type=0;
      po->amount=amount;
   }
      break;
   }
   
   status=MF_STATUS_ADJUSTMENT;
   Post(&hdr, false);
}

void CMfPosEngine::Discrepency(int type, double amount)
{
   BhPosHdr		   hdr;
   BhDiscrepancy *dis=(BhDiscrepancy *) hdr.data;
   
   memset(&hdr, 0, sizeof(hdr));
   
   dis->ID;
   dis->flags;
   dis->type=type;
   dis->amount=amount;
   strcpy(dis->extra_data, "");
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_DISCREPANCY;
   hdr.hdr.subcmd=type;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhDiscrepancy);
   
   //SetStatus(POST_STATUS_ACK);
   Post(&hdr, false);
}

void CMfPosEngine::ztape()
{
	BhPosHdr	  hdr;

	memset(&hdr, 0, sizeof(hdr));

	hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_SERVICE;
   hdr.hdr.subcmd=BHMSG_POST_SERVICE_SUMMARY_ZTAPE;
   hdr.hdr.Ex=0;
   hdr.hdr.size=0;
	
	status=MF_STATUS_ZTAPE;
   Post(&hdr, false);
}

void CMfPosEngine::GetTransSummary()
{
   BhPosHdr      hdr;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_SERVICE;
   hdr.hdr.subcmd=BHMSG_POST_SERVICE_SUMMARY_TRANS;
   hdr.hdr.Ex=0;
   hdr.hdr.size=0;
   
   status=MF_STATUS_TRANSSUM;
   Post(&hdr, false);
}

void CMfPosEngine::GetUserSummary()
{
   BhPosHdr      hdr;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_SERVICE;
   hdr.hdr.subcmd=BHMSG_POST_SERVICE_SUMMARY_USER;
   hdr.hdr.Ex=0;
   hdr.hdr.size=0;
   
   status=MF_STATUS_USERSUM;
   Post(&hdr, false);
}

void CMfPosEngine::BlindBalance(int flags)
{
   BhPosHdr				hdr;
   
   memset(&hdr, 0, sizeof(hdr));
   
   BhBlindBalance *bb=(BhBlindBalance *) hdr.data;
   bb->flags=0;
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_BLINDBALANCE;
   hdr.hdr.subcmd=BHMSG_POST_BLINDBALANCE_TRANSAMOUNT;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhBlindBalance);
   
   status=MF_STATUS_BLINDBALANCE;
   Post(&hdr, false);
}

void CMfPosEngine::Interface(int type, const char *data)
{
	BhPosHdr	  hdr;
	BhInterfaceRequestBarcode *bcif=(BhInterfaceRequestBarcode *) hdr.data;

	memset(&hdr, 0, sizeof(hdr));

	strncpy(bcif->barcode, data, 15);

	hdr.hdr.type=BHPOS_TYPE_SYSTEM;
	hdr.hdr.cmd=BHMSG_SYSTEM_INTERFACE;
	hdr.hdr.subcmd=type;
	hdr.hdr.size=sizeof(bcif);
	
	status=MF_STATUS_IFACK;
   Post(&hdr, false);
}

}
