/*
saleitem.cpp - a point of sale client class
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2004 Al Kingsley
*/

#include "config.h"
#include <fcntl.h>

#include "pos.h"
#include "bhnetwork.h"
#include "ctransitems.h"
#include "cmfengine.h"

namespace bhmfengine {

int CMfPosEngine::NumTrans()
{
   bool bServerSide;
   config.GetOption(MFENG_CONFIG_TRANSSERVERSIDE, &bServerSide);
   
   int n=transstack.NumTrans();
   transaction_signal.emit(BHMF_EVENT_SYSTEM_NUMTRANS, &n);
}

int CMfPosEngine::TransList()
{
   bool bServerSide;
   config.GetOption(MFENG_CONFIG_TRANSSERVERSIDE, &bServerSide);
   
   //transstack.NumTrans();
}

int CMfPosEngine::TransCycle(bhpos::CTransItems& ct)
{
   int i=-1, num;
   bhpos::BhTransactionType type;
   bhpos::BhTransStatus     status;
   
   if((num=transstack.NumTrans())==0){ // do nothing
      return -1;
   }
   
   g_print("\nmfeng: numt:%d  curr:%d\n", num, i, current_index);
   current_index=transstack.GetNext(ct);
   if(current_index < -1){
      return current_index;
   }
   if(current_index==-1){
      current_transaction=-1;
      return -1;
   }
   current_transaction=ct.GetID();
   type=ct.GetType();
   status=ct.GetStatus();
   return current_index;
}

int CMfPosEngine::TransVoid()
{
   BhPosHdr            hdr;
   BhTransactionCancel tc;
   
   memset(&hdr, 0, sizeof(hdr));
   memset(&tc, 0, sizeof(tc));
   tc.transID=current_transaction;
   //tc.type=type;
   tc.type=0;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_CANCEL;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(tc);
   memcpy(hdr.data, &tc, sizeof(tc));
   status=MF_STATUS_VOIDACK;
#ifdef _DEBUUGERY
   g_print("cancel tid:%d\n", tc.transID);
#endif   
   Post(&hdr, true);
}

int CMfPosEngine::TransCommit()
{
   BhPosHdr hdr;
	double	z;
   bhpos::ChargeSummary chargesummary;
	static bhpos::CPayItem pitem;
	static bool bCredit;
   bhpos::CTransItems trans;
   bhpos::BhTransactionType tctype=transstack.GetType(current_transaction);
   
   transstack.Get(current_transaction, trans);
	
	memset(&hdr, 0, sizeof(hdr));
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   
   if(trans.GetType()==bhpos::BHTRANSACTION_TYPE_EPHEMERAL){
      EphemeralCommit();
   }
   
   //trans.SetType(bhpos::BHTRANSACTION_TYPE_TRANSACTION);
   
   //g_print("tcstatus: %d\n", tcstatus);
   
   switch(tcstatus){
   case MF_STATUS_TRANSCOMMIT:
   {
		pitem.Clear();
		bCredit=false;
      BhTransCommit tc;
      
      memset(&tc, 0, sizeof(BhTransCommit));
      
      hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_COMMIT;
      hdr.hdr.Ex=0;
      hdr.hdr.size=sizeof(tc);
      
      trans.Rationalize();
      
      bhpos::HeaderPrint phdr;
				
		phdr.transID=trans.GetID();
		phdr.UserID=userID;
		phdr.StationID=connection_info.PostID;

      CashDrawer(0);		
		
		tm		*Time;
		long	 CurrentLocalTime;
		time_t  CurrentTime;
				
		CurrentLocalTime = time(&CurrentTime);
		Time = localtime(&CurrentLocalTime);
		strftime(phdr.datetime, sizeof(phdr.datetime), "%x", Time);
		char buff[8];
		sprintf(buff, " - %02d:%02d", Time->tm_hour, Time->tm_min);
		strcat(phdr.datetime, buff);
      Print(bhpos::PRNCMD_PREPRINT, NULL, 0);
      Print(bhpos::PRNCMD_HEADER, &phdr, sizeof(phdr));
      
      tc.transID=trans.GetID();
      if(tctype==bhpos::BHTRANSACTION_TYPE_DELIVERY){
         tc.delivery=1;
         tc.customerID=trans.GetCustomerID();
      }
      tc.total_items=trans.NumAllItemsQty();
      tc.total_unknownitems=trans.NumUnknownItems();
      tc.total_returnitems=trans.NumReturnItems();
      tc.total_rentalitems=trans.NumRentalItems();
      tc.total_laybyitems=trans.NumLaybyItems();
      tc.total_payment_types=trans.NumPayment();
      tc.total_tax=trans.GetNumber(bhpos::BHNUM_TAX);
      tc.total_discount=trans.GetNumber(bhpos::BHNUM_DISCOUNT);
      tc.change=trans.GetNumber(bhpos::BHNUM_CHANGE);
      tc.total=trans.GetNumber(bhpos::BHNUM_TOTAL);
      /*
      printf("\n\nNumber of Items:%d\n",tc.total_items);
      printf("Number of unknown Items:%d\n",tc.total_unknownitems);
      printf("Number of return Items:%d\n",tc.total_returnitems);
      printf("Number of rental Items:%d\n",tc.total_rentalitems);
      printf("Number of payment Items:%d\n",tc.total_payment_types);
      printf("change:%d\n",tc.change);
      printf("total :%f\n\n",tc.total);*/
      memcpy(hdr.data, &tc, sizeof(tc));
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_TRANSCOMMIT;
      
      memset(&transsum, 0, sizeof(transsum));
      transsum.NumItems=tc.total_items;
      transsum.Subtotal=trans.GetNumber(bhpos::BHNUM_SUBTOTAL);
      transsum.Discount=tc.total_discount;
      transsum.Tax=tc.total_tax;
   
      Post(&hdr, true);
      
      return 0;
   }
      break;
   
   case MF_STATUS_PLUCOMMIT:
      if(CommitPlu()) return 1;

   case MF_STATUS_RETURNPLUCOMMIT:
      if(CommitReturnPlu()) return 1;

   case MF_STATUS_RENTALPLUCOMMIT:
      if(CommitRentalPlu()) return 1;

   case MF_STATUS_LAYBYPLUCOMMIT:
      if(CommitLaybyPlu()) return 1;

   case MF_STATUS_UPLUCOMMIT:
      if(CommitUPlu()) return 1;

   case MF_STATUS_PRICECOMMIT:
      if(CommitPricePlu()) return 1;
   
   case MF_STATUS_PAYCOMMIT:
   {
      if(tctype!=bhpos::BHTRANSACTION_TYPE_DELIVERY && trans.NumPayment()!=0){
         int                   tmp_numpay=trans.NumPayment();
         BhTransPayment       *ptrpay;
         BhTransPaymentCommit *paycommit;
         bhpos::CPayItem       pi;
      
         paycommit=(BhTransPaymentCommit *) hdr.data;
         ptrpay=paycommit->payment;
         paycommit->transID=trans.GetID();
      
         hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_COMMIT_PT;
         hdr.hdr.Ex=0;
         hdr.hdr.size=sizeof(BhTransPaymentCommit) - sizeof(BhTransPayment);
			
			Print(bhpos::PRNCMD_TRANSSUMMARY, &transsum, 0);
      
			if(tmp_numpay==1 && pi.GetFlags()==bhpos::PTYPE_FLAG_DONE){
				Print(bhpos::PRNCMD_PAYITEM, &pi, 1);
			}
			else if(tmp_numpay!=0){
            while((float) (1024 / (sizeof(BhTransPayment) * tmp_numpay)) < 1.0) tmp_numpay--;
	         paycommit->num_items=tmp_numpay;
   	      while(tmp_numpay){
      	      trans.PopPayment(pi);
					Print(bhpos::PRNCMD_PAYITEM, &pi,trans.NumPayment());
					if(pi.GetID()==bhpos::PTYPE_CREDIT){
						pitem=pi;
						bCredit=true;
					}
					if(pi.GetFlags()==bhpos::PTYPE_FLAG_DONE){
						tmp_numpay--;
						continue;
					}
					ptrpay->type=pi.GetID();
	            ptrpay->flags=pi.GetFlags();
   	         ptrpay->amount=pi.GetAmount();
      	      ptrpay++;
         	   tmp_numpay--;
            	//trans.CashDrawerAdjust(pi.GetID(), pi.GetAmount());
               if(pi.GetID()==bhpos::PTYPE_CASH){
	               // check if there is money in drawer
   	            // if so remove some
      	         // cashd.Remove(pi.GetAmount());
         	   }
               if(pi.GetID()==bhpos::PTYPE_INHOUSECHARGE){
                  // aquire charge balance to print on rec
               }
            	hdr.hdr.size+=sizeof(BhTransPayment);
            }
            transstack.Update(current_transaction, trans);
	         status=MF_STATUS_ACK;
   	      tcstatus=MF_STATUS_PAYCOMMIT;
      	   Post(&hdr, true);
         	return 0;
			}
      }
      tcstatus=MF_STATUS_TRANSEND;
      
		z=trans.GetNumber(bhpos::BHNUM_CHANGE);
		if(z!=0.0){
			Print(bhpos::PRNCMD_CHANGE, &z,0);
		}
   }
   
   case MF_STATUS_TRANSEND:
   {
      BhTransEnd tc;
      bool bExternCredit;
      config.GetOption(MFENG_CONFIG_EXTERNCREDIT, &bExternCredit);
      
      if(tctype==bhpos::BHTRANSACTION_TYPE_DELIVERY){
         //Print(bhpos::PRNCMD_TRANSSUMMARY, &transsum, 0);
      }
      
      tc.transID=trans.GetID();
      hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_END;
      memcpy(hdr.data, &tc, sizeof(tc));
      hdr.hdr.size=sizeof(BhTransEnd);
      status=MF_STATUS_ACK;
      //if(bHaveCharge){
         //memcpy(&chargesummary, &chargebal, sizeof(chargebal));
      	//Print(bhpos::PRNCMD_CHARGE, (void *) &chargesummary,0);
		//}
      if(bCredit){
         if(!bExternCredit){
      	   Print(bhpos::PRNCMD_CREDIT, (void *) &pitem, 0);
         }
		}
      Print(bhpos::PRNCMD_FOOTER, NULL, 0);
		Print(bhpos::PRNCMD_POSTPRINT, NULL, 0);
      Print(bhpos::PRNCMD_CDISPPOSTSALE, NULL, 0);
		
      Post(&hdr, true);
   }
   }
   
   return 0;
}

int CMfPosEngine::EphemeralCommit()
{
   BhPosHdr      hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   memset(&hdr, 0, sizeof(hdr));
        
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_EPHEMERAL;
   hdr.hdr.Ex=0;
      
   switch(tcstatus){
   case MF_STATUS_TRANSCOMMIT:
      BhTransEphemCommit      tc;
      
      memset(&tc, 0, sizeof(BhTransEphemCommit));
      
      hdr.hdr.subcmd=BHMSG_POST_EPHEMERAL_COMMIT;
      hdr.hdr.size=sizeof(tc);
      
      trans.Rationalize();
      
      tc.num_items=trans.NumItems();
      memcpy(hdr.data, &tc, sizeof(tc));
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_TRANSCOMMIT;
      Post(&hdr, true);
      
      return 0;
 
   case MF_STATUS_PLUCOMMIT:
   {
      int                tmp_numplu=(trans.NumAllItems() - trans.NumUnknownItems());
      bhpos::CSaleItem   si;
      hdr.hdr.subcmd=BHMSG_POST_EPHEMERAL_PLU;
      hdr.hdr.Ex=0;
      hdr.hdr.size=sizeof(BhTransEphemPluCommit) - sizeof(BhTransEphemItem);
   
      BhTransEphemPluCommit   *plucommit=(BhTransEphemPluCommit *) hdr.data;
      
      if(tmp_numplu > 0){
         while((float) (1024 / (sizeof(BhTransItem) * tmp_numplu)) < 1.0) tmp_numplu--;
         plucommit->num_items=tmp_numplu;
         BhTransEphemItem *ptrplu=plucommit->items;
         while(tmp_numplu){
            trans.Pop(si);
            strcpy(ptrplu->PLU, si.GetPLU().data());
            ptrplu->flags=0;
#ifdef _DEBUGGERY
            g_print("Ephem PLU commit: popped %s - wrote: %s, %d, messize:%d\n", si.GetPLU(), ptrplu->PLU, ptrplu->flags, hdr.hdr.size+sizeof(BhTransItem));
#endif    
            ptrplu++;
            tmp_numplu--;
            hdr.hdr.size+=sizeof(BhTransEphemItem);
         }
         transstack.Update(current_transaction, trans);
         status=MF_STATUS_ACK;
         tcstatus=MF_STATUS_PLUCOMMIT;
         Post(&hdr, true);
         
         return 1;
      }
      // no items left
      tcstatus=MF_STATUS_TRANSEND;
   }
         
   case MF_STATUS_TRANSEND:
      hdr.hdr.subcmd=BHMSG_POST_EPHEMERAL_END;
      hdr.hdr.size=0;
      status=MF_STATUS_ACK;
      Post(&hdr, true);
      break;
   }
   return 0;
}

int CMfPosEngine::CommitPlu()
{
   BhPosHdr           hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numplu=trans.NumItems();
   bhpos::CSaleItem   si;
   
   memset(&hdr, 0, sizeof(hdr));
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_COMMIT_PLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransPluCommit) - sizeof(BhTransItem);
   
   BhTransPluCommit   *plucommit=(BhTransPluCommit *) hdr.data;
   plucommit->transID=trans.GetID();
      
   if(tmp_numplu > 0){      
      while((float) (1024 / (sizeof(BhTransItem) * tmp_numplu)) < 1.0) tmp_numplu--;
      plucommit->num_items=tmp_numplu;
      BhTransItem *ptrplu=plucommit->items;
      while(tmp_numplu){
         if(trans.Pop(si)){
            Print(bhpos::PRNCMD_ITEM, &si, 0);
            strcpy(ptrplu->PLU, si.GetPLU().data());
            ptrplu->flags=si.GetFlags();
            ptrplu->type=si.GetType();
            ptrplu->quantity=si.GetQuantity();

#ifdef _DEBUGGERY
            g_print("PLU commit: popped %s - wrote: %s, %d, %d\nnumleft:%d\nmessize:%d\n", si.GetPLU(), ptrplu->PLU, ptrplu->flags, ptrplu->quantity, trans.NumAllItems(), hdr.hdr.size+sizeof(BhTransItem));
#endif    
            ptrplu++;
            tmp_numplu--;
            hdr.hdr.size+=sizeof(BhTransItem);
         }
         else{
          tcstatus=MF_STATUS_RETURNPLUCOMMIT;
          return 0;        
         }
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_PLUCOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
      // no items left
   tcstatus=MF_STATUS_RETURNPLUCOMMIT;
   return 0;
}

int CMfPosEngine::CommitReturnPlu()
{
   BhPosHdr           hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numplu=trans.NumReturnItems();
   bhpos::CSaleItem   si;
   
   memset(&hdr, 0, sizeof(hdr));
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_RETURNS_PLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransPluCommit) - sizeof(BhTransItem);
   
   BhTransPluCommit   *plucommit=(BhTransPluCommit *) hdr.data;
   plucommit->transID=trans.GetID();
      
   if(tmp_numplu > 0){      
      while((float) (1024 / (sizeof(BhTransItem) * tmp_numplu)) < 1.0) tmp_numplu--;
      plucommit->num_items=tmp_numplu;
      BhTransItem *ptrplu= plucommit->items;
      while(tmp_numplu){
         if(trans.PopReturn(si)){
            Print(bhpos::PRNCMD_ITEM, &si, 0);
            strcpy(ptrplu->PLU, si.GetPLU().data());
            ptrplu->flags=si.GetReason();
            ptrplu->type=si.GetType();
            ptrplu->quantity=si.GetQuantity();

#ifdef _DEBUGGERY
            g_print("Return PLU commit: popped %s - wrote: %s, %d, %d\nnumleft:%d\nmessize:%d\n", si.GetPLU(), ptrplu->PLU, ptrplu->flags, ptrplu->quantity, trans.NumAllItems(), hdr.hdr.size+sizeof(BhTransItem));
#endif    
            ptrplu++;
            tmp_numplu--;
            hdr.hdr.size+=sizeof(BhTransItem);
         }
         else{
            tcstatus=MF_STATUS_RENTALPLUCOMMIT;
            return 0;        
         }
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_RETURNPLUCOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
      // no items left
   tcstatus=MF_STATUS_RENTALPLUCOMMIT;
   return 0;
}

int CMfPosEngine::CommitRentalPlu()
{
   BhPosHdr           hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numplu=trans.NumRentalItems();
   bhpos::CSaleItem   si;
   
   memset(&hdr, 0, sizeof(hdr));
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_RENTAL_PLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransPluCommit) - sizeof(BhTransItem);
   
   BhTransPluCommit   *plucommit=(BhTransPluCommit *) hdr.data;
   plucommit->transID=trans.GetID();
      
   if(tmp_numplu > 0){      
      while((float) (1024 / (sizeof(BhTransItem) * tmp_numplu)) < 1.0) tmp_numplu--;
      plucommit->num_items=tmp_numplu;
      BhTransItem *ptrplu=plucommit->items;
      while(tmp_numplu){
         if(trans.PopRental(si)){
            Print(bhpos::PRNCMD_ITEM, &si, 0);
            strcpy(ptrplu->PLU, si.GetPLU().data());
            ptrplu->flags=si.GetFlags();
            ptrplu->type=si.GetType();
            ptrplu->quantity=si.GetQuantity();

#ifdef _DEBUGGERY
            g_print("Rental PLU commit: popped %s - wrote: %s, %d, %d\nnumleft:%d\nmessize:%d\n", si.GetPLU(), ptrplu->PLU, ptrplu->flags, ptrplu->quantity, trans.NumAllItems(), hdr.hdr.size+sizeof(BhTransItem));
#endif    
            ptrplu++;
            tmp_numplu--;
            hdr.hdr.size+=sizeof(BhTransItem);
         }
         else{
            tcstatus=MF_STATUS_LAYBYPLUCOMMIT;
            return 0;        
         }
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_RENTALPLUCOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
      // no items left
   tcstatus=MF_STATUS_LAYBYPLUCOMMIT;
   return 0;
}

int CMfPosEngine::CommitLaybyPlu()
{
   BhPosHdr           hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numplu=trans.NumLaybyItems();
   bhpos::CSaleItem   si;
   
   memset(&hdr, 0, sizeof(hdr));
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_LAYBY_PLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransPluCommit) - sizeof(BhTransItem);
   
   BhTransPluCommit   *plucommit=(BhTransPluCommit *) hdr.data;
   plucommit->transID=trans.GetID();
      
   if(tmp_numplu > 0){      
      while((float) (1024 / (sizeof(BhTransItem) * tmp_numplu)) < 1.0) tmp_numplu--;
      plucommit->num_items=tmp_numplu;
      BhTransItem *ptrplu=plucommit->items;
      while(tmp_numplu){
         if(trans.PopLayby(si)){
            Print(bhpos::PRNCMD_ITEM, &si, 0);
            strcpy(ptrplu->PLU, si.GetPLU().data());
            ptrplu->flags=si.GetFlags();
            ptrplu->type=si.GetType();
            ptrplu->quantity=si.GetQuantity();

#ifdef _DEBUGGERY
            g_print("Layby PLU commit: popped %s - wrote: %s, %d, %d\nnumleft:%d\nmessize:%d\n", si.GetPLU(), ptrplu->PLU, ptrplu->flags, ptrplu->quantity, trans.NumAllItems(), hdr.hdr.size+sizeof(BhTransItem));
#endif    
            ptrplu++;
            tmp_numplu--;
            hdr.hdr.size+=sizeof(BhTransItem);
         }
         else{
            tcstatus=MF_STATUS_UPLUCOMMIT;
            return 0;        
         }
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_LAYBYPLUCOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
      // no items left
   tcstatus=MF_STATUS_UPLUCOMMIT;
   return 0;
}

int CMfPosEngine::CommitUPlu()   
{
   BhPosHdr      hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numuplu=trans.NumUnknownItems();
   BhTransItemUPLU   *ptruplu;
   BhTransUPluCommit *uplucommit;
   bhpos::CSaleItem   si;

   memset(&hdr, 0, sizeof(hdr));
   uplucommit=(BhTransUPluCommit *) hdr.data;      
   uplucommit->transID=trans.GetID();
   ptruplu=uplucommit->items;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_COMMIT_UPLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransUPluCommit) - sizeof(BhTransItemUPLU);

#ifdef _DEBUGGERY   
   g_print("CPost::UPLU commit num: %d\n", tmp_numuplu);
#endif
      
   if(tmp_numuplu > 0){
      while((float) (1024 / (sizeof(BhTransItemUPLU) * tmp_numuplu)) < 1.0) tmp_numuplu--;
      uplucommit->num_items=tmp_numuplu;
      while(tmp_numuplu){
         if(!trans.PopUnknown(si)){
            tcstatus=MF_STATUS_PRICECOMMIT;
            return 0;
         }
         Print(bhpos::PRNCMD_ITEM, &si, 0);
         
         strcpy(ptruplu->PLU, si.GetPLU().data());
         ptruplu->flags=si.GetFlags();
         ptruplu->quantity=si.GetQuantity();
         ptruplu->unitPrice=si.GetPrice();
         ptruplu->tax=si.GetTax();
         strncpy(ptruplu->description, si.GetDispText().data(), 63);
         ptruplu->description[63]=0;   // just in case
         
         //g_print("UPLU commit: %f\n", si.GetTax());
         //g_print("UPLU commit: %s:%f:%s\n", ptruplu->PLU, ptruplu->unitPrice, ptruplu->description);
         
         ptruplu++;
         tmp_numuplu--;
         hdr.hdr.size+=sizeof(BhTransItemUPLU);
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_UPLUCOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
   // no items left
   tcstatus=MF_STATUS_PRICECOMMIT;
   return 0;
}

int CMfPosEngine::CommitPricePlu()   
{
   BhPosHdr      hdr;
   bhpos::CTransItems trans;
   transstack.Get(current_transaction, trans);
   int                tmp_numuplu=trans.NumPriceItems();
   BhTransItemUPLU   *ptruplu;
   BhTransUPluCommit *uplucommit;
   bhpos::CSaleItem   si;

   memset(&hdr, 0, sizeof(hdr));
   uplucommit=(BhTransUPluCommit *) hdr.data;      
   uplucommit->transID=trans.GetID();
   ptruplu=uplucommit->items;
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_COMMIT_PRICEPLU;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhTransUPluCommit) - sizeof(BhTransItemUPLU);

#ifdef _DEBUGGERY   
   g_print("CPost::UPLU commit num: %d\n", tmp_numuplu);
#endif
      
   if(tmp_numuplu > 0){
      while((float) (1024 / (sizeof(BhTransItemUPLU) * tmp_numuplu)) < 1.0) tmp_numuplu--;
      uplucommit->num_items=tmp_numuplu;
      while(tmp_numuplu){
         if(!trans.PopPrice(si)){
            tcstatus=MF_STATUS_PAYCOMMIT;
            return 0;
         }
         Print(bhpos::PRNCMD_ITEM, &si, 0);
         
         strcpy(ptruplu->PLU, si.GetPLU().data());
         ptruplu->flags=si.GetFlags();
         ptruplu->quantity=si.GetQuantity();
         ptruplu->unitPrice=si.GetPrice();
         ptruplu->tax=si.GetTax();
         strncpy(ptruplu->description, si.GetDispText().data(), 63);
         ptruplu->description[63]=0;   // just in case
         ptruplu++;
         tmp_numuplu--;
         hdr.hdr.size+=sizeof(BhTransItemUPLU);
      }
      transstack.Update(current_transaction, trans);
      status=MF_STATUS_ACK;
      tcstatus=MF_STATUS_PRICECOMMIT;
      Post(&hdr, true);
         
      return 1;
   }
   // no items left
   tcstatus=MF_STATUS_PAYCOMMIT;
   return 0;
}

int CMfPosEngine::TransNoSale()
{
   BhPosHdr hdr;
   
   bool bNoSale;
   config.GetOption(MFENG_CONFIG_DRAWERPOLICY, &bNoSale);
   if(!bNoSale) return 1;
   
   memset(&hdr, 0, sizeof(hdr));
   
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.size=0;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_NOSALE;
   Post(&hdr, false);
   status=MF_STATUS_NOSALE;
   CashDrawer(0);
   return 0;
}

int CMfPosEngine::TransID()
{
   status=MF_STATUS_TRANSID;
}

int CMfPosEngine::TransTender()
{
}

void CMfPosEngine::ReturnReason()
{
	BhPosHdr	  hdr;

	memset(&hdr, 0, sizeof(hdr));

	hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_LIST;
   hdr.hdr.subcmd=BHMSG_POST_LIST_RETURNREASON;
   hdr.hdr.Ex=0;
   hdr.hdr.size=0;
	
	status=MF_STATUS_RETURNREASON;
   Post(&hdr, false);
}

void CMfPosEngine::CashbackCheck(int type, double amount)
{
   BhPosHdr		   hdr;
   BhCashbackCheck *cbc=(BhCashbackCheck *) hdr.data;
   
   memset(&hdr, 0, sizeof(hdr));
      
   cbc->type=type;
   cbc->amount=amount;
   hdr.hdr.type=BHPOS_TYPE_POST;
   hdr.hdr.cmd=BHMSG_POST_TRANSACTION;
   hdr.hdr.subcmd=BHMSG_POST_TRANSACTION_CASHBACKCHK;
   hdr.hdr.Ex=0;
   hdr.hdr.size=sizeof(BhCashbackCheck);
   
   status=MF_STATUS_CASHBACKCHECK;
   Post(&hdr, false);
}

}
